(in-package :http)

(defmacro define-gd ((&key (func-name (gentemp "GD-"))
			   (url (format nil "/~(~a~)" func-name))
			   (class :user)
			   (code :ok))
		     lambda-list &body forms)
  "This macro defines a function that, when called by the server, generates an image in gif format
using the GD library.
func-name   - the name of the defined function
url         - the url of the function 
code        - the http response code associated with the page
class       - class of the page, :user, :system, or a class name
lambda-list - parameters of the defined function
forms       - body of the function. The last evaluated form must return the GD image."
  
  (let ((gd-name (gensym)))
    `(progn
       (set-symbol-info ',func-name :page ,url ',class)
       (defun ,func-name ,lambda-list
	 (let ((,gd-name (progn ,@forms)))
	   (http-send-image ,gd-name)
	   (gd-image-destroy ,gd-name)))
       ,(when url
	  `(register-page ,url ',func-name ',class))
       ',func-name)))

(defun http-send-image (image)
  (let ((fd (socket-fd *http-stream*)))
    (gd-image-fd image fd)
    (force-output *http-stream*)))

;;; Animations
;;; An animation is a function with the following characteristics:
;;;   It is initially called with the set of arguments defined in spec
;;;   It should return a list (image new-args...)
;;;   It should return no new-args when the animation is over

(defun http-send-animation (spec)
  (let ((fd (socket-fd *http-stream*))
	(fun (car spec))
	(args (cdr spec))
	(boundary "BoundaryString"))
					;(format *http-stream* (animation-header boundary))
					;(force-output *http-stream*)
    (http-header *http-animation-header* :boundary boundary)
    (setq boundary (concatenate 'string "--" boundary))
    (loop 
      (unless args			;finished?
	(return))
      (format *http-stream* "~%~a~%Content-Type: image/gif~%~%" boundary)
      (force-output *http-stream*)
      (setq args (apply fun args))
      (gd-image-fd (car args) fd)
      (setq args (cdr args)))
    (format *http-stream* "~%~a~%" boundary)
    (force-output *http-stream*)))

