(in-package :lispweb-user)

(define-application doc
    :class :system
    :url "/doc"
    :pretty-name  "LispWeb Documentation"
    :link nil)

(define-page ("Server features" :func-name features-page :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "Server Features")))
  (html-out "<P>The ~a server is an HTTP server entirely written in Common Lisp (apart from a few low-level functions in C). " (bold "LispWeb"))
  (html-out "It was designed to enable Lisp applications to run over the network, and to use a standard Web browser as their user interface. ")
  (html-out "The functions that define an application should produce HTML code as their output; they are loaded into the running image ~
of the server, and are called by the server when necessary.~%")
  (html-out "<P>The main features of this server are:~%")
  (html-out (dotted-list
	     (list (html-build "~a - the server handles HTML files, images, image maps, GET and POST forms and the PUT method. 
It is also interfaced with the ~a sendmail protocol.<p>" (bold "Standard functionalities") (italic "SMTP"))
		   (html-build "~a - every URL that resides on the server can be mapped to a Lisp function 
that dynamically generates the HTML code for the requested page. The function can take arguments that are derived from specific components 
of the requested URL. This applies to text pages, images (through the ~a package), forms and image maps.<p>" 
			       (bold "Dynamical generation of HTML documents")
			       (anchor "http://aim.unipv.it/misc/gd.html" "Gd-lisp"))
		   (html-build "~a - a set of Common Lisp functions and macros is provided to generate 
syntactically correct HTML code. In this way it is possible to take advantage of both the power of Lisp and the universality of HTML. 
For more information, see the ~a.<p>" 
			       (bold "HTML authoring in Lisp")
			       (page-anchor 'hall "HALL on-line documentation"))
		   (html-build "~a - the ~a allows you to interactively control the server parameters and the underlying Lisp environment.<p>" 
			       (bold "Interactive server control")
			       (page-anchor 'status-page "Setup page" :full t))
		   (html-build "~a - the server can keep a record of all the requests received, and of the time needed to answer to each request.<P>" 
			       (bold "Logging facilities")))))
  (html-out "Just to be fair, take a look also at the ~a." (page-anchor 'bugs-page "bug list")))

(define-page ("LispWeb Known Bugs" :func-name bugs-page :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "Server Bugs and Missing Features<P>")))
  (html-out "<P>This is a list of known bugs and missing functionalities of the ~a server.~%" (bold "LispWeb"))
  (html-out (dotted-list
	     (list (html-build "Since the server is not stateless, pages tend to get out of date very quickly. In other words, using the ~a button is potentially dangerous because it can bring up old pages that are no longer consistent with the current state of the server. I don't think there's anything to do about this problem...<P>" (italic "Back"))
		   "Think about adding memory management to the control page.<P>"
		   "A few items of the HALL on-line documentation don't work.<P>"
		   "The HALL library must be checked against some HTML standard. The use of quotes should be made consistent.")))
  (html-out "Feel free to give your ~a on any of the above listed problems. " (page-anchor 'comment-page "opinion or contribution"))
  (html-out "On the other hand, the server also has some interesting ~a." (page-anchor 'features-page "features")))

(define-page ("LispWeb Documentation" :func-name lispweb-doc :url "/top" :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "LispWeb Documentation")))
  (html-out (header 3 
		    (dotted-list (list (page-anchor 'manifest-page "The LispWeb Manifest")
				       (page-anchor 'features-page "Server features")
				       (html-build "Hall documentation: ~a"
						   (dotted-list (list (page-anchor 'pagedef "Page definition")
								      (page-anchor 'style "Page styles")
								      (page-anchor 'auth "Authorization management")
								      (page-anchor 'hall "Library overview")
								      (page-anchor 'forms "Form management")
								      (anchor "/doc/imaps" "Image maps")
								      (page-anchor 'marlowe "Marlowe (Page source viewer)")
								      (page-anchor 'applications "Application development")
								      (page-anchor 'js-lib "JavaScript library")
								      (page-anchor 'examples-top "Examples" :full t))))
				       "Server configuration parameters"
				       (page-anchor 'http-info "HTTP information")
				       (page-anchor 'bugs-page "Bugs and missing functionalities"))))))

(define-page ("LispWeb Manifest" :func-name manifest-page :class doc)
    ()
  (html-out (include "manifest.html"))
  (html-out "<P>The ideas discussed above have led to the development of ~a, a specialized ~a server
with support for the creation of ~a and for ~a."
	    (anchor "/" (bold "LispWeb"))
	    (code "HTTP")
	    (page-anchor 'hall "dynamical documents")
	    (page-anchor 'stsp "server-to-server communication"))
  (html-out *copyright*))

(define-page ("HTTP information" :func-name http-info :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "HTTP Info<P>")))
  (format-http-info))

(define-page ("HTML Authoring Lisp Library: page definition" :func-name pagedef :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "HTML Authoring Lisp Library")))
  (html-out (header 3 "Page definition"))
  (html-out "<P>To define an HTML page, you should use the ~a macro, whose exact syntax is the following:" (bold "define-page"))
  (html-out (preformatted
	     "<P>(define-page ((<i>title</i> &key <i>func-name
                          url
                          code
                          meta
                          frames
                          class
                          package
                          background
		          bgcolor
			  text
			  link 
			  vlink
			  alink
			  auth
                          style
                          visible</i>)
               <i>lambda-list</i>)
  &body <i>forms</i>)"))
  (html-out 
   (in-table (:border 0)
	     (html-out (table-row (table-header "title" :valign :top :align :left)
				  (table-cell (html-build "A string that represents the title of the page; it is automatically included 
at the beginning of the page between ~a.~%" (fixed "&lt;title&gt; ... &lt;/title&gt;")))))
	     (html-out (table-row (table-header "func-name" :valign :top :align :left)
				  (table-cell (html-build "The name of the lisp function that will be generated by the ~a form. 
You should supply this in order to be able to associate the page with the corresponding URL.~%" (bold "define-page")))))
	     (html-out (table-row (table-header "url" :valign :top :align :left)
				  (table-cell (html-build "A string that represents the URL of the page. It defaults to ~a with a 
prepended \"/\".~%" (italic "func-name")))))
	     (html-out (table-row (table-header "code" :valign :top :align :left)
				  (table-cell (html-build "The response code sent by the server in the header of the page. 
It defaults to ~a (200).~%" (bold ":ok")))))
	     (html-out (table-row (table-header "meta" :valign :top :align :left)
				  (table-cell (html-build "A list of ~a specifications. It should be a list of lists containing keyword-value pairs.
Allowed keywords are ~a, ~a and ~a.~%" (fixed "meta") (bold ":http-equiv") (bold ":content") (bold ":url")))))
	     (html-out (table-row (table-header "frames" :valign :top :align :left)
				  (table-cell (html-build "A ~a specification.~%" (fixed "frameset")))))
	     (html-out (table-row (table-header "class" :valign :top :align :left)
				  (table-cell (html-build "Can be ~a (the default value), ~a, or the name of an ~a. ~%"
							  (bold ":user") 
							  (bold ":system") 
							  (page-anchor 'applications "application")))))
	     (html-out (table-row (table-header "package" :valign :top :align :left)
				  (table-cell (html-build "The package the function should run in.~%"))))
	     (html-out (table-row (table-header "background" :valign :top :align :left)
				  (table-cell (html-build "Name of a ~a image to use as the background for the page.~%" (bold "gif")))))
	     (html-out (table-row (table-header "bgcolor" :valign :top :align :left)
				  (table-cell (html-build "The background color.~%"))))
	     (html-out (table-row (table-header "text" :valign :top :align :left)
				  (table-cell (html-build "The text color.~%"))))
	     (html-out (table-row (table-header "link" :valign :top :align :left)
				  (table-cell (html-build "The link color.~%"))))
	     (html-out (table-row (table-header "vlink" :valign :top :align :left)
				  (table-cell (html-build "The visited link color.~%"))))
	     (html-out (table-row (table-header "alink" :valign :top :align :left)
				  (table-cell (html-build "The active link color.~%"))))
	     (html-out (table-row (table-header "auth" :valign :top :align :left)
				  (table-cell (html-build "The ~a for this page.~%" (anchor "/doc/auth" "authorization list")))))
	     (html-out (table-row (table-header "style" :valign :top :align :left)
				  (table-cell (html-build "A ~a structure to use for this page.~%" 
							  (anchor "/doc/style" "page style")))))
	     (html-out (table-row (table-header "visible" :valign :top :align :left)
				  (table-cell (html-build "Set to ~a if you want the source code for this page to be visible through ~a.~%"
							  (bold "T") (anchor "/doc/marlowe" "Marlowe")))))
	     (html-out (table-row (table-header "lambda-list" :valign :top :align :left)
				  (table-cell (html-build "The lambda list of the function you are defining."))))))
  (html-out "<P>The body of the function can contain any arbitrary lisp form; however, to produce HTML code you should 
use the ~a macro.~%"
	    (bold "html-out"))
  (html-out "Its syntax is ~a; it is essentially the same as the ~a function without the ~a argument (because the output is always sent to the client over the network).~%" (fixed "(html-out format-string &rest arguments)") (bold "format") (italic "stream")))

(define-page ("HTML Authoring Lisp Library: page styles" :func-name style :class doc)
    ()
  (html-out *LispWeb-banner*)
  ;(html-out (center (header 1 "HTML Authoring Lisp Library")))
  (html-out (center (header 1 "Page styles")))
  (html-out "If you want to create several pages with the same style, you can create a ~a structure and pass it to ~a.~%" 
	    (bold "page-style")
	    (bold "define-page"))
  (html-out "The structure is created using the ~a function:~%" (bold "define-page-style"))
  (html-out (preformatted
	     "<P>(define-page-style (&key <i>code
			 meta
			 background
			 bgcolor
			 text
			 link
			 vlink
			 alink</i>))"))
  (html-out "<P>All keyword arguments supplied to this function will be applied to the defined page. Here is an example:~%")
  (html-out (preformatted "<P>(setq *alarm-style* (define-page-style :bgcolor (color-string :red 200) 
                                       :text (color-string :red 250 :green 250)))"))
  (html-out (preformatted "<P>(define-page (\"Alarm!\" :url \"/alarm\" :class :system :style *alarm-style*)
    ()
  (html-out (center (header 1 \"Alarm!\"))))")) 
  (html-out "Click ~a to see the result.~%" (anchor "/alarm" "here"))
  (html-out "<P>You can use ~a to merge two different styles; the resulting style will be the logical ~a of the two arguments. This means that it 
will contain all options appearing on the two original styles, with the first one taking precedence if both specify a value for the same option.~%"
	    (bold "merge-page-styles") (bold "or")))

(defvar *alarm-style* (define-page-style :bgcolor (color-string :red 200) 
			:text (color-string :red 250 :green 250)))

(define-page ("Alarm!" :url "/alarm" :class :system :style *alarm-style*)
    ()
  (html-out (center (header 1 "Alarm!"))))

(define-page ("HTML Authoring Lisp Library: authorizations" :func-name auth :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "Authorization lists")))
  (html-out "You can define an ~a for a page. Only hosts that match one of the entries in the list are allowed to view the page.~%" 
	    (italic "authorization list"))
  (html-out "Each entry in the authorization list is a partial or complete IP address represented as a list of integers. 
You can replace the tail of the address with a ~a to match a whole sub-domain. For example, the following are valid list items:~%" 
	    (bold "*"))
  (html-out (preformatted "(193 204 34 6)"))
  (html-out (preformatted "(193 204 34 *)"))
  (html-out (preformatted "(193 204 *)"))
  (html-out (preformatted "(193 *)"))
  (html-out "<P>You can use the ~a function to build an authorization item from the string representation of the address 
(that can be obtained with ~a). For example:~%"
	    (bold "make-auth-list")
	    (bold "get-host-by-addr"))
  (html-out (preformatted "(make-auth-list (get-host-by-addr \"ipvaim.unipv.it\")) --> (193 204 34 6)")))
  
(define-page ("HTML Authoring Lisp Library: form management" :func-name forms :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "Form management")))
  (html-out "The ~a macro allows to easily access the values of the fields of a fill-in form. Used in the body of an ~a function, it allows to bind
ordinary lisp variables to the values in the form fields.~%" (bold "with-form-fields") (fixed "ACTION"))
  (html-out "For example, this is the function that implements the action for the ~a:~%" (page-anchor 'comment-page "comment page" :full t))
  (html-out "~a" (preformatted "<P>(define-page (\"Thank You!\" :func-name thanks-page)
    ()
  (with-form-fields (realname email comments)
    (send-mail \"LispWeb-user\" 
	       *operator-address* 
	       (format nil \"~a (~a from ~a) writes:~%~%~a\" realname
		       email
		       (http-info-addr *http-info*)
		       comments))
    (html-out \"Comments received from ~a (~a). Thank you!~%\" realname email)
    (html-out (header 4 \"Go back to the ~a~%\" (anchor \"/\" \"top page\")))))"))
  (html-out "<P>~a, ~a and ~a are the names of the three text fields appearing in the ~a." 
	    (bold "realname")
	    (bold "email")
	    (bold "comments")
	    (page-anchor 'comment-page "comment page" :full t)))

(define-page ("HTML Authoring Lisp Library: defining applications" :func-name applications :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "HTML Authoring Lisp Library")))
  (html-out (header 3 "Applications"))
  (html-out "Applications are defined using the ~b macro:~%" (bold "define-application"))
  (html-out "~a" (preformatted 
"<P>(define-application (name 
		     &key class
		          url
		          pretty-name
		          link))"))
  (html-out 
   (in-table (:border 0)
	     (html-out (table-row (table-header "name" :valign :top :align :left)
				  (table-cell "The name of the application.")))
	     (html-out (table-row (table-header "class" :valign :top :align :left)
				  (table-cell "The class to which this application belongs.")))
	     (html-out (table-row (table-header "url" :valign :top :align :left)
				  (table-cell "The common prefix of the URLs of the pages belonging to this application.")))
	     (html-out (table-row (table-header "pretty-name" :valign :top :align :left)
				  (table-cell "Human-readable name of the application")))
	     (html-out (table-row (table-header "link" :valign :top :align :left)
				  (table-cell "The top-level link for this application. This will appear in the top page of the server.")))
	     ))
  (html-out "<P><br>Pages that belong to an application should specify the application's ~a as their ~a argument.~%" 
	    (italic "name") 
	    (bold ":class"))
  )

(define-page ("JavaScript library" :func-name js-lib :class doc)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "JavaScript Library")))
  (html-out "The following constants store JavaScript functions that you can include and use in your pages:~%")
  (html-out (dotted-list (mapcar #'(lambda (name)
				     (page-anchor 'js-lib-view name :args (list name)))
				 *LispWeb-js-library*))))


(define-page ("JavaScript library" :func-name js-lib-view :class doc)
    (name)
  (let ((js-function (intern name :http)))
    (html-out *LispWeb-banner*) 
    (html-out (center (header 1 name)))
    (html-out (preformatted (symbol-value js-function)))))

;;; Examples

(define-application examples
    :class doc
    :url "/examples"
    :link nil)

(define-page ("HTML Authoring Lisp Library: examples" :func-name examples-top :url "/top" :class examples)
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "HTML Authoring Lisp Library")))
  (html-out (header 3 "Examples"))
  (html-out "This is an example of a page-defining lisp form. It displays the same information in two different ways, according to
the argument passed as the second component of the path (see below).")
  (html-out "~a" (plaintext 
"(define-page (\"Example Page\" :func-name example-page :url \"/examples/pink\" :class :system :visible t)
    (request)
  (cond ((string= request \"image\")
	 (html-out (inline-image \"floyd.gif\")))
	((string= request \"text\")
	 (html-out (center (header 2 \"Pink Floyd<BR>The Dark Side of<BR>the Moon\")))
	 (html-out (center (in-table ()
				     (html-out (table-row (table-header 
							   \"Speak To Me - Breathe - On The Run<br>
							   Time - The Great Gig In The Sky<br>
							   Money - Us And Them<br>
							   Any Colour You Like<br>
							   Brain Damage - Eclipse<br>
							   [)(] Dolby system\")))))))))"))
  (html-out "Now you can try this page by calling it as ~a or ~a." (anchor "/doc/examples/pink/image" "/doc/examples/pink/image") 
	    (anchor "/doc/examples/pink/text" "/doc/examples/pink/text"))
  (html-out "<HR>~%")
  (html-out "<P>The following form defines a page that counts how many times it was accessed.")
  (html-out "~a" (plaintext 
"(define-page (\"A page with a counter\" :url \"/examples/counter\"
				      :class :system
				      :visible t)
    ()
  (incf *access-counter*)
  (html-out \"This page was accessed ~a times.\" (bold *access-counter*)))" ))
  (html-out (anchor "/doc/examples/counter" "Just try it!"))
  (html-out "<HR>~%")
  (html-out "<P>This example shows how the combination of partial
URLs and argument passing allows to hold state information. The following page displays the value of a numerical variable, and
offers the user the chance to increment or decrement it.")
  (html-out "~a" (plaintext 
"(define-page (\"A page with an argument\" :func-name arg-page
                                        :url \"/examples/args\"
                                        :class :system
                                        :visible t)
    (&optional (state 0))
  (setq state (read-from-string state))
  (html-out \"The current value of the state is ~a.\" (bold state))
  (html-out \"You can ~a it or ~a it.\"
            (anchor (html-build \"/examples/args/~d\" (1+ state))
                    \"increase\")
            (anchor (html-build \"/examples/args/~d\" (1- state))
                    \"decrease\")))"))
  (html-out "Try this page with ~a!~%" (anchor "/doc/examples/args/100" "state = 100")))

(defvar *access-counter* 0)

(define-page ("A page with a counter" :func-name counter-page
				      :url "/counter"
				      :class examples
				      :visible t)
    ()
  (incf *access-counter*)
  (html-out "This page was accessed ~a times." (bold *access-counter*)))

(define-page ("A page with an argument" :func-name arg-page
                                        :url "/args"
					:class examples
					:visible t)
    (&optional (state 0))
  (setq state (read-from-string state))
  (html-out "The current value of the state is ~a." (bold state))
  (html-out "You can ~a it or ~a it."
            (page-anchor 'arg-page "increase" :args (list (1+ state)))
            (page-anchor 'arg-page "decrease" :args (list (1- state)))))
  
(define-page ("HTML Authoring Lisp Library" :func-name hall :class doc)
    (&optional request)
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "HTML Authoring Lisp Library")))
  (if request
      (let* ((sym (intern (string-upcase request) :http))
	     (demo-form (second (assoc sym *LispWeb-hall-demo*
				       :test #'(lambda (key entry)
						 (cond ((stringp entry) nil)
						       ((listp entry)
							(member key entry))
						       (t (eq key entry))))))))
	(html-out "This example shows the use of ~a~%" (bold request))
	(html-out "<P>This Lisp form:<P>~(~a~)~%" (preformatted (plaintext (write-to-string (list 'html-out demo-form) :pretty t))))
	(html-out "<P>Produces the following HTML code:<P>~a~%" (plaintext (eval demo-form)))
	(html-out "<P>On your web browser, it looks like this:<P>~a~%" (eval demo-form))
	(html-out "<HR>~a~%" (page-anchor 'hall "Back to the main list" :full t)))
    (progn
      (html-out "HALL is a set of Common Lisp functions and macros that you can use to produce HTML code from within your lisp programs. ~%")
      (html-out (header 3 "Library overview"))
      (html-out "<P>This is a list of all the functions you can use in your HTML-producing Lisp forms. ~
Click on them to see an example of their usage.~%")
      (html-out "<TABLE>~%")
      (dolist (item *LispWeb-hall-demo*)
	(let ((sym (car item)))
	  (if (stringp sym)
	      (html-out (html-build "<TR><TH COLSPAN=2><BR>~a~%" sym))
	    (if (listp (car item))
		(mapc #'(lambda (x)
			  (html-out "<TR><TD>~(~a~)<TD>~(~a~)~%" 
				    (page-anchor 'hall x :args (list (car sym)))
				    (italic (format nil "~a" (hall-arglist x)))))
		      sym)
	      (html-out "<TR><TD>~(~a~)<TD>~(~a~)~%" 
			(page-anchor 'hall sym :args (list sym))
			(italic (format nil "~a" (hall-arglist sym))))))))
      (html-out "</TABLE>"))))

(define-page ("Example Page" :func-name example-page :url "/pink" :class examples :visible t)
    (request)   ; take arguments
  (cond ((string= request "image")
	 (html-out (center (inline-image "floyd.gif"))))
	((string= request "text")
	 (html-out (center (header 2 "Pink Floyd<BR>The Dark Side of<BR>the Moon")))
	 (html-out (center (in-table ()
			     (html-out (table-row (table-header 
"Speak To Me - Breathe - On The Run<br>
 Time - The Great Gig In The Sky<br>
 Money - Us And Them<br>
Any Colour You Like<br>
Brain Damage - Eclipse<br>
[)(] Dolby system")))))))))

;;; Javascript

(define-page ("Javascript Test" :func-name js-test :class doc :visible t
				:javascript (list *js-is-empty*
						  *js-is-date*
						  *js-is-number*
						  *js-in-bounds*))
    ()
  (html-out *LispWeb-banner*)
  (html-out (center (header 1 "Javascript Test")))
  (html-out (in-form nil
		     (html-out "This field should not be empty: ~a<P>~%"
			       (text-field "jst1" 
					   :onchange "if (is_empty(this.value)) alert('Please fill this field!')"
					   :onfocus "window.status='Please enter something!'"))
		     (html-out "This field should contain a date: ~a<P>~%"
			       (text-field "jst2" 
					   :onchange "if (!is_date(this.value)) alert('Please enter a date in the format dd/mm/yy!')"
					   :onfocus "window.status='Please enter a date!'"))
		     (html-out "This field should contain a number: ~a<P>~%"
			       (text-field "jst3" 
					   :onchange "if (!is_number(this.value)) alert('Please enter a number!')"
					   :onfocus "window.status='Please enter a number!'"))
		     (html-out "This field should contain a number between 0 and 100: ~a<P>~%"
			       (text-field "jst4" 
					   :onchange "if (!in_bounds(this.value, 0, 100)) alert('Please enter a number between 0 and 100!')"
					   :onfocus "window.status='Please enter a number between 0 and 100!'"))
		     )))

;;; -*- Marlowe Page -*-

(define-page ("Marlowe's Page" :func-name marlowe :class doc :visible t)
    () 
  (html-out (center (header 2 "Marlowe")))
  ;(blueline 500)
  (html-out (italic "As client programs - such as Netscape, Mosaic, Java, and
so on - are able to display the source of the current HTML page, a Lisp
Server should able to display the Lisp code behind the current page.
Lisp programmers would probably call such a thing an HTML Inspector.
We prefer to think it is more like a Private Eye, and we call it Marlowe."))
  ;(blueline 500)
  (html-out "<p>Marlowe is a tool developed by the ~a. On this Server, 
  most pages have a small icon in the lower right corner depicting 
  the famous Private Eye. If you click on it, you will automatically 
  retrieve the Lisp source which generated the page. This can be useful
  to debug your pages, to learn how to make them better, or just to show
  off what a good Lisp programmer you are." 
	    (anchor "http://kmi.open.ac.uk" "Knowledge Media Institute"))
  (html-out "<p>To activate the Marlowe feature, specify ~a when defining a page with ~a.~%"
	    (bold ":visible t") (anchor "/doc/pagedef" (bold "define-page")))
  (html-out "<p>Now, try Marlowe by clicking on the icon in the lower right
corner of this page and retrieve the Lisp source of this page."))

;;; Services

(define-page ("Loaded services" :func-name services :class doc)
    ()
  (html-out *Lispweb-banner*)
  (html-out "These are the services loaded in the server:")
  (html-out (blockquote (dotted-list (mapcar #'http::service-link *LispWeb-services*)))))

;;; Cookies 

(define-page ("Netscape Cookies" :func-name cookie-man-1 :class doc :visible t)
    ()
  (html-out *LispWeb-banner*)
  (html-out (header 2 "Cookie generator"))
  (html-out "Current cookies:")
  (html-out (dotted-list (header-values :cookie)))
  (html-out "Please insert the name and the value of the cookie you want to create in the fields below:")
  (html-out (in-form (:action (url 'cookie-ghost))
		     (html-out (center
				(in-table (:border 0)
					  (html-out (table-row
						     (table-header "Name:")
						     (table-cell (text-field "name"))))
					  (html-out (table-row
						     (table-header "Value:")
						     (table-cell (text-field "value")))))))
		     (html-out "Press ~a to create the cookie or ~a to reset this form."
			       (submit-button "Ok")
			       (reset-button "Reset")))))

(define-no-page (:func-name cookie-ghost :class doc)
    ()
  (with-field-value (name value)
    (let ((*my-cookie* (http::create-cookie name value)))
      (declare (special *my-cookie*))
      (cookie-man-2 *my-cookie*))))
      
(define-page ("Netscape Cookies" :func-name cookie-man-2 :class doc :visible t :cookie *my-cookie*)
    (cookie)
  (html-out *LispWeb-banner*)
  (html-out (header 2 "Cookie generator"))
  (html-out "You have created a cookie named ~a whose value is ~a."
	    (bold (http::cookie-name cookie))
	    (bold (http::cookie-value))))

