(in-package :http)

;;; Experimental

(defmacro define-file-tunnel (name &key url class auth)
  `(progn
     (defun ,name (path &rest ignore)
       (declare (ignore ignore))
       ,(when auth
	  `(check-auth ,auth))		;this does not return if auth check fails
       (let ((true-path (concatenate 'string ,url path)))
	 (let ((ext (subseq true-path (1+ (position #\. true-path :from-end t)))))
	   (if (string= ext "htm")
	       (include-binary-file true-path "text/html")
	     (include-binary-file true-path (format nil "image/~a" ext)))))
       t)
     (setf (get ',name :lispweb-fun-type) :dispatch)
     (register-page ,url ',name ',class)))

#|
(defun include-binary-file (path type)
  (let ((buf (make-array 1024 :element-type 'character))
	(ret 0))
    (with-open-file (in path :element-type 'character :if-does-not-exist nil)
      (if in
          (progn
            (http-header *http-generic-header* :type type :length (file-length in))
					;(format *http-stream* (generic-header type ;(format nil "image/~a" type)
					;(file-length in)))
					;(force-output *http-stream*)
					;(unless (eq (http-info-method *http-info*)
					;'head)
            (loop 
	     (setq ret 
                   #+ALLEGRO   (read-sequence buf in)
                   #+LISPWORKS (common-lisp::read-sequence buf in)
	           #+LUCID     (read-array buf in)
	           #+CLISP     (read-char-sequence buf in) )
	     (when (zerop ret)
	       (return))
	     #+ALLEGRO   (write-sequence buf *http-stream* :end ret)
             #+LISPWORKS (common-lisp::write-sequence buf *http-stream* :end ret)
	     #+LUCID     (write-array buf *http-stream* :end ret)
	     #+CLISP     (write-char-sequence buf *http-stream* :end ret)
	     ))
        (http-header *http-text-header* :code-name :not-found)))))
|#

;(defun include-binary-file (path type)
;  (with-open-file (stream path :if-does-not-exist nil)
;    (if stream
;	(progn
;	  (http-header *http-generic-header* :type type :length (file-length stream))
;	  (do ((el (read-char stream nil nil) (read-char stream nil nil)))
;	      ((null el))
;	    (write-char el *http-stream*)))
;      (http-header *http-text-header* :code-name :not-found))))

;;;try faster version
;(defun include-binary-file (path type)
;  (let ((buf (make-array 1024 :element-type 'character))
;	(ret 0))
;    (with-open-file (in path :element-type 'character :if-does-not-exist nil)
;      (if in
;          (progn
;            (http-header *http-generic-header* :type type :length (file-length in))
;            (loop 
;	     (setq ret 
;                   #+ALLEGRO   (read-sequence buf in)
;                   #+LISPWORKS (common-lisp::read-sequence buf in)
;	           #+LUCID     (read-array buf in)
;	           #+CLISP     (read-char-sequence buf in) )
;	     (when (zerop ret)
;	       (return))
;	     #+ALLEGRO   (write-sequence buf *http-stream* :end ret)
;             #+LISPWORKS (common-lisp::write-sequence buf *http-stream* :end ret)
;	     #+LUCID     (write-array buf *http-stream* :end ret)
;	     #+CLISP     (write-char-sequence buf *http-stream* :end ret)
;	     ))
;          (http-header *http-text-header* :code-name :not-found)))))


;;;john domingue thinks there's a problem writing 2 java files at once

(defvar *change-binary-queue-lock*
    (mp:make-lock :name "binary-queue-lock"))

(defvar *write-binary-file-process-queue* nil)

;(defun include-binary-file (path type)
;  (mp:with-lock (*change-binary-queue-lock*)
;    (setf *write-binary-file-process-queue*
;	  (nconc *write-binary-file-process-queue*
;	         (list mp::*current-process*))))
;  (when (not (eq mp::*current-process* (car *write-binary-file-process-queue*)))
;    ;;(format t "~%waiting ~a ~a" *host* pathname)
;    (mp::process-wait "somone writing binary file"
;                      #'(lambda () (eq mp::*current-process*
;                                       (car *write-binary-file-process-queue*))))
;    ;;(format t "~%out ~a ~a" *host* pathname)
;    )
;  (unwind-protect
;      (with-open-file (stream path :if-does-not-exist nil)
;        (if stream
;	    (progn
;	      (http-header *http-generic-header* :type type :length (file-length stream))
;	      (do ((el (read-char stream nil nil) (read-char stream nil nil)))
;	          ((null el))
;	        (write-char el *http-stream*)))
;            (http-header *http-text-header* :code-name :not-found)))
;    (mp:with-lock (*change-binary-queue-lock*)
;      (setf *write-binary-file-process-queue*
;            (remove mp::*current-process*
;		    *write-binary-file-process-queue*)))))

;;;merge my locked version with alberto's faster one
(defun albertos-include-binary-file (path type)
  ;;(format t "albertos-include-binary-file ~a ~a~%" path type)
  (let ((buf (make-array 1024 :element-type 'character))
	(ret 0))
    (with-open-file (in path :element-type 'character :if-does-not-exist nil)
      (if in
          (progn
            (http-header *http-generic-header* :type type :length (file-length in))
            (loop 
	     (setq ret 
                   #+ALLEGRO   (read-sequence buf in)
                   #+LISPWORKS (common-lisp::read-sequence buf in)
	           #+LUCID     (read-array buf in)
	           #+CLISP     (read-char-sequence buf in) )
	     (when (zerop ret)
	       (return))
	     #+ALLEGRO   (write-sequence buf *http-stream* :end ret)
             #+LISPWORKS (common-lisp::write-sequence buf *http-stream* :end ret)
	     #+LUCID     (write-array buf *http-stream* :end ret)
	     #+CLISP     (write-char-sequence buf *http-stream* :end ret)
	     ))
          (http-header *http-text-header* :code-name :not-found)))))

(defun old-include-binary-file (path type)
 ;; (format t "~a ~a" path type)
 ;;(push (format nil "old-include-binary-file ~a ~a" path type) f)
  (with-open-file (in path :element-type 'unsigned-byte  :if-does-not-exist nil)
    (if in
        (progn
          (http-header *http-generic-header* :type type :length (file-length in))
          (do ((el (read-byte in nil nil) (read-byte in nil nil)))
              ((null el))
            (write-char (code-char el) *http-stream*)))
      (http-header *http-text-header* :code-name :not-found))))

(defun include-binary-file (path type)
  (mp:with-lock (*change-binary-queue-lock*)
    (setf *write-binary-file-process-queue*
	  (nconc *write-binary-file-process-queue*
	         (list mp::*current-process*))))
  (when (not (eq mp::*current-process* (car *write-binary-file-process-queue*)))
    ;;(format t "~%waiting ~a ~a" *host* pathname)
    (mp::process-wait "somone writing binary file"
                      #'(lambda () (eq mp::*current-process*
                                       (car *write-binary-file-process-queue*))))
    ;;(format t "~%out ~a ~a" *host* pathname)
    )
  (unwind-protect
      ;;use this one fo pc lispworks because write-sequence isn't defined
      #+harlequin-pc-lisp
    (old-include-binary-file path type)
    #+harlequin-unix-lisp
    (albertos-include-binary-file path type)
    (mp:with-lock (*change-binary-queue-lock*)
      (setf *write-binary-file-process-queue*
            (remove mp::*current-process*
		    *write-binary-file-process-queue*)))))

;(define-file-tunnel cdrom-tunnel :url "/cdrom" :class :user :auth '((193 204 34 *)))
