;;; Mode: Lisp; Package: http

;;; Author: John Domingue

;;; The Open University

(in-package "HTTP")

;;some useful additions

(defun find-variable-start (x &optional (start 0))
  (or (find #\? x :start start)
      (find #\& x :start start)))

(defun position-variable-start (x &optional (start 0))
  (if (find #\? x :start start)
      (position #\? x :start start)
      (position #\& x :start start)))

(defun find-value-start (x &optional (start 0))
  (find #\= x :start start))

(defun position-value-start (x &optional (start 0))
  (position #\= x :start start))

(defvar *whitespace-chars*
  '(#\space #\tab #\linefeed #\return))

(defun get-decoded-form-value (form name &optional string-name
                                    (read-p t))
  (unless (zerop (length form))
  (unless string-name
    (setf string-name (string-downcase
                       (if (symbolp name)
                           (symbol-name name)
                           (format nil "~a" name)))))
  (let* ((start (and (find-variable-start form)
                     (position-variable-start form))))
    (when start
      (setf form (subseq form start))
      (when (> (length form) 1)
        (setf form (string-trim *whitespace-chars*
                                (subseq form 1)))
        (let ((value-start (and (find-value-start form)
                                (position-value-start form))))
          (when value-start
            (let ((variable-name 
                   (string-trim *whitespace-chars*
                                (subseq form 0 value-start))))
              (if (string= string-name 
                           (string-downcase variable-name))
                  (let ((variable-start 
                         (and (find-variable-start form value-start)
                              (position-variable-start
                               form value-start)))
                        value-string)
                  (if variable-start
                      (setf value-string
                       (string-upcase (subseq form (1+ value-start)
                                              variable-start)))
                    (setf value-string (subseq form (1+ value-start))))
                  (if read-p
                      (when (> (length 
                                (string-trim 
                                 *whitespace-chars* value-string)) 
                               0)
                        (read-from-string value-string))
                    value-string))
                (get-decoded-form-value (subseq form value-start)
                                        name string-name))))))))))

(defun get-decoded-form-string-value (form name)
  (get-decoded-form-value form name
                          (string-downcase
                       (if (symbolp name)
                           (symbol-name name)
                           (format nil "~a" name)))
                          nil))

(defun cl-user::get-decoded-form-value (form name)
  (get-decoded-form-value form name))

(defun empty-string-p (string)
  (zerop (length string)))