;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-


(in-package "OCML")

(ensure-ontology gen-design method 
                   "OCML:PHD-LIBRARY;methods;gen-design;load2.lisp")


(def-ontology a-star-design
    :includes (gen-design))

(in-ontology a-star-design)

(def-class a-star-DESIGN-method (design-method)
  ((has-input-role :value has-h-cost-function)
   (has-h-cost-function :type cost-function))
  :lisp-slots ((design-state-type :initform 'a-star-design-state)
               )
  :lisp-class-name a-star-DESIGN-method)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass a-star-design-state (design-state)
  ((estimated-cost :accessor state-estimated-cost :initform nil)
   (total-cost :accessor state-total-cost :initform nil)))


(defmethod print-object ((state a-star-design-state) stream)
  (with-slots (depth estimated-cost design-extensions cost)state
    (format stream "#<STATE ~{ ~S~}; ~s| est:~S; cost: ~s>" 
            design-extensions depth estimated-cost cost)))

(defmethod DESIGN-FROM-STATE (appl (method a-star-design-method) (state a-star-design-state))
  (extend-evaluate-select-control appl method state))


(defmethod EVALUATE-STATE-internal :after (appl (method a-star-design-method)(state a-star-design-state))
  (setf (state-cost state) (evaluate-state-cost appl method state)
        (state-estimated-cost state)
        (estimate-distance-to-goal-state appl method state)
        (state-total-cost state)
        (compute-state-total-cost appl method state)))

(defmethod cheaper-state? ((state1 a-star-design-state)(state2 a-star-design-state))
  (let ((c1 (state-total-cost state1))
        (c2 (state-total-cost state2)))
    (cheaper-cost? c1 c2)))
  
(defmethod INTERNAL-ADD-STATE ((method a-star-design-method) (state a-star-design-state))
  (incf *state-counter*)
  (cond ((null *design-space*)
         (setf *design-space* (cons state *design-space*)))
        (t
         (let ((i (print (position state *design-space*
                            :test #'(lambda (new-state old-state)
                                      (not (cheaper-state? old-state new-state )))))))
           (if i
             (setf *design-space*
                   (append (subseq *design-space* 0 i)
                           (List state)
                           (nthcdr i *design-space*)))
             (setf *design-space* (append *design-space* (List state)))))))

  (if (> (length *design-space*)2)
    (pprint (subseq  *design-space* 0 2))
     (pprint *design-space*)))
             

(defmethod estimate-distance-to-goal-state (appl (method a-star-design-method)
                                                 (state a-star-design-state))
   appl ;;;ignore
  (with-slots (design-model) state
    (ocml-call (the-slot-value (name method) 'has-h-cost-function)
              (list 'quote design-model))))

(defmethod compute-state-total-cost (appl (method a-star-design-method)
                                          (state a-star-design-state))
   appl ;;;ignore
  (with-slots (estimated-cost cost) state
    (add-costs estimated-cost cost)))

  

