;;; Mode: Lisp; Package: ocml

;;; File created in WebOnto

(in-package "OCML")

(in-ontology aktive-portal-services)


(def-rule person-services-rule
    ((person-details ?person 
                     (?appellation ?full-name ?job-title ?email-address 
                                   ?web-address
                                   ?organisation ?research-interest))
     if
     (= ?full-name
        (setofall ?x (full-name ?person ?x)))
     (= ?email-address
        (setofall ?x (has-email-address ?person ?x)))
     (= ?appellation
        (setofall ?x (has-appellation ?person ?x)))
     (= ?job-title
        (setofall ?x (has-job-title ?person ?x)))
     (= ?organisation
        (setofall ?x (works-for ?person ?x)))
     (= ?web-address
        (setofall ?x (has-web-address ?person ?x)))
     (= ?research-interest
        (setofall ?x (has-research-interest ?person ?x)))))

(def-rule person-works-with-rule
    ((works-with ?person ?partner-full-name ?partner-url ?family-name)
     if
     (has-project-member ?project ?person)
     (has-project-member ?project ?partner)
     (full-name ?partner ?partner-full-name)
     (family-name ?partner ?family-name)
     (has-web-address ?partner ?partner-url)
     (not (== ?person ?partner)))
    ((works-with ?person ?partner-full-name "http://www.aktors.org/" ?family-name)
     if
     (has-project-member ?project ?person)
     (has-project-member ?project ?partner)
     (full-name ?partner ?partner-full-name)
     (family-name ?partner ?family-name)
     (not (has-web-address ?partner ?partner-url))
     (not (== ?person ?partner))))

#|too slow with southampton people
(def-rule working-person-shares-person-interests-with-rule
    ((shares-interests-with working-person ?person 
                            ?shared-interest-person-full-name ?shared-interest)
     if
     (has-research-interest ?person ?interest1)
     (has-research-interest ?shared-interest-person ?interest2)
     (not (== ?person ?shared-interest-person))
     (related-interests ?interest1 ?interest2 ?shared-interest)
     (full-name ?shared-interest-person ?shared-interest-person-full-name)))
|#

(def-rule person-shares-person-interests-with-rule
    ((shares-interests-with ?person 
                            ?shared-interest-person-full-name ?shared-interest 
                            ?shared-interest-person-url)
     if
     (has-research-interest ?person ?shared-interest)
     (has-research-interest ?shared-interest-person ?shared-interest)
     (not (== ?person ?shared-interest-person))
     (full-name ?shared-interest-person ?shared-interest-person-full-name)
     (has-web-address ?shared-interest-person ?shared-interest-person-url)))


(def-rule related-interests-rule 
    ((related-interests ?interest1 ?interest1 ?interest1))
    ((related-interests ?interest1 ?interest2 ?x)
     (sub-area-of ?interest1 ?x)
     (sub-area-of ?interest2 ?x)))

(def-rule person-related-stories-rule
    ((related-stories person ?person ?headline)
     if
     (relates-events ?story ?event)
     (has-main-agent ?event ?person)
     (has-headline ?story ?headline))
    ((related-stories person ?person ?headline)
     if
     (relates-events ?story ?event)
     (has-other-agents-involved ?event ?person)
     (has-headline ?story ?headline))
    ((related-stories project ?project ?headline)
     if
     (relates-events ?story ?event)
     (project-involved ?event ?project)
     (has-headline ?story ?headline))
    ((related-stories project ?project ?headline)
     if
     (relates-events ?story ?event)
     (has-award-rationale ?event ?project)
     (has-headline ?story ?headline)))

(def-rule people-web-page-rule
    ((person-web-page person ?person ?url)  
     if 
     (person ?person)
     (has-web-address ?person ?url)))


(def-rule people-current-projects-rule
    ((current-project ?person ?project ?project-url)  
     if 
     (has-project-member ?project ?person)
     (has-web-address ?project ?project-url))
    ((current-project ?person ?project ?project-url)  
     if 
     (has-project-leader ?project ?person)
     (has-web-address ?project ?project-url)))



;;(def-relation current-project (?person ?project)
;;  :argument-types (person project))


(def-rule people-interesting-projects-rule
    ((project-might-be-interested-in  ?person (?project ?shared-areas))
     if 
     (= ?all-person-research-areas 
        (setofall ?y (has-research-interest ?person ?y)))
    (person ?person)
    (project ?project)
     (= ?all-project-research-areas 
        (setofall ?x (addresses-generic-area-of-interest ?project ?x)))
    (not (has-project-member ?project ?person))
     (not (has-project-leader ?project ?person))
    (= ?shared-areas (intersection ?all-project-research-areas ?all-person-research-areas))
     (= ?l (length ?shared-areas))
     (> ?l 0)
     ))

(def-relation project-might-be-interested-in (?person ?shared-areas)
  :argument-types (person list))


(def-rule person-number-of-projects-rule
    ((person-number-of-projects ?person ?n)
     if
     (= ?p1 (setofall ?project (has-project-leader ?project ?person)))
     (= ?p2 (setofall ?project (has-project-member ?project ?person)))
     (= ?n (+ (length ?p1) (length ?p2)))))

(def-relation person-number-of-projects (?person ?n)
  :argument-types (person integer))

(def-rule person-number-of-research-areas-rule
    ((person-number-of-research-areas ?person ?n)
     if
     (= ?r (setofall ?research-area (has-research-interest ?person ?research-area)))
     (= ?n (length ?r))))

(def-relation person-number-of-research-areas (?person ?n)
  :argument-types (person integer))

(def-rule person-kpopularity-of-research-areas-rule
    ((person-kpopularity-of-research-areas ?person ?n)
     if
     (= ?pops (setofall ?pop
                     (and (has-research-interest ?person ?research-area)
                          (kpopularity ?research-area ?pop))))
     (= ?n (apply '+ ?pops))))


(def-rule person-kpopularity-and-number-of-research-areas-rule
    ((person-kpopularity-and-number-of-research-areas ?person ?pop ?n)
     if
     (= ?pops (setofall ?pop
                     (and (has-research-interest ?person ?research-area)
                          (kpopularity ?research-area ?pop))))
     (= ?pop (apply '+ ?pops))
     (= ?n (length ?pops))))

(def-rule person-log-popularity-of-research-areas-rule
    ((person-log-popularity-of-research-areas ?person ?n)
     if
     (= ?pops (setofall ?pop
                     (and (has-research-interest ?person ?research-area)
                          (log-popularity ?research-area ?pop))))
     (= ?n (apply '+ ?pops))))

(def-relation person-log-popularity-of-research-areas (?person ?n)
  :argument-types (person integer))

(def-rule person-log-popularity-and-number-of-research-areas-rule
    ((person-log-popularity-and-number-of-research-areas ?person ?pop ?n)
     if
     (= ?pops (setofall ?pop
                     (and (has-research-interest ?person ?research-area)
                          (log-popularity ?research-area ?pop))))
     (= ?pop (apply '+ ?pops))
     (= ?n (length ?pops))))

(def-rule person-combined-workplace-name
    ((combined-workplace-name ?person ?workplace-name)
     if
     (works-for ?person ?organisation)
     (works-in-unit ?person ?unit)
     (= ?workplace-name (concatenate-names ?organisation ?unit))))

(def-class combined-workplace-name-type () ())

(def-relation combined-workplace-name (?person ?workplace-name)
  :argument-types (person combined-workplace-name-type))

(def-rule person-combined-workplace-name-remove-the
    ((combined-workplace-name-remove-the ?person ?workplace-name)
     if
     (works-for ?person ?organisation)
     (works-in-unit ?person ?unit)
     (= ?workplace-name (concatenate-names-remove-the ?organisation ?unit))))



(defun concatenate-names (symbol &rest symbols)
  (intern (format nil "~a~{-~a~}" (symbol-name symbol) (mapcar #'symbol-name symbols))
          (find-package "OCML")))

(def-function concatenate-names (?symbol &rest ?symbols)
  "Creates a new symbol which concantenates the given arguments"
   :lisp-fun #'concatenate-names)


(defun concatenate-names-remove-the (symbol &rest symbols)
  (let ((string (format nil "~a~{-~a~}"
                        (symbol-name symbol) (mapcar #'symbol-name symbols))))
    (if (and (> (length string) 4) (string= "THE-" (subseq string 0 4)))
        (intern (subseq string 4) (find-package "OCML"))
      (intern string (find-package "OCML")))))

(def-function concatenate-names-remove-the (?symbol &rest ?symbols)
  "Creates a new symbol which concantenates the given arguments. Removes a leading 'the'"
   :lisp-fun #'concatenate-names-remove-the)

