;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-

(in-package "OCML")

(in-ontology gen-design)



(def-relation POSSIBLE-VALUE (?p ?v)
  :constraint (parameter ?p))

(def-relation DESIGN-FOCUS-ORDER (?x ?y)
  "This relation can be used to specify an application specific
   ordering of design foci"
  :constraint (and (design-focus ?x)
                   design-focus ?y))


(tell (DEFINES-PARTIAL-ORDER design-focus-order)) 


(def-function SELECT-MOST-PREFERRED-FOCUS (?l ?rel)
  "The most preferred focus is one which is in 
   the input list (i.e. which is a possibility in the 
   current scenario) such that there is no other focus 
   which is preferred to it"
  :body (the ?focus 
          (and (member ?focus ?l)
               (not (exists ?focus2
                            (and (member ?focus2 ?l)
                                 (<> ?focus2 ?focus)
                                 (holds ?rel ?focus2 ?focus)))))))

(def-function COLLECT-MOST-RESTRICTED-PARAMETERS (?l ?rel)
  :body (in-environment 
         ((?pairs . (sort (map '(lambda (?p)
                                  (list-of 
                                   ?p (setofall 
                                       ?v (holds ?rel ?p ?v))))
                               ?l)
                          '(kappa (?x ?y)
                            (< (length (second ?x))
                               (length (second ?y)))))))
         (map first (filter 
                     ?pairs 
                     '(kappa (?pair)
                       (= (second ?pair)
                          (second (first ?pairs))))))))