;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-


(in-package "OCML")



(in-ontology hill-climbing)


(def-class hc-DESIGN-method (design-method)
   :lisp-slots ((design-state-type :initform 'hc-design-state))
  :lisp-class-name hc-DESIGN-method)

(defclass hc-design-state (design-state)
  ())

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmethod DESIGN-FROM-STATE (appl (method  HC-DESIGN-METHOD) (state hc-design-state))
  (if (violated-constraints state)
    (state-fails state)
    (if (state-feasible? state)
      (if (state-complete? state)
        (design-succeeds state)
        (expand-state appl method state))
      (state-fails state))))

(defmethod expand-state (appl (method  HC-DESIGN-METHOD) (state hc-design-state))
  ;;;(setf *design-space* nil)  ;;;add this line to prevent backtracking
  (extend-design appl method state)
  ;;;(setf (state-foci state) nil)
  (loop for result = (retry-design-operator appl method 
                                            (current-operator state) state 
                                            (current-focus state) :extend)
        while result))

(defmethod SELECT-DESIGN-FOCUS (appl (method  HC-DESIGN-METHOD) (state design-state) 
                                     context)
     appl method context
     (car (state-foci state)))

(defmethod SORT-OPERATORS (appl (method  HC-DESIGN-METHOD) 
                                (state design-state)
                                ops)
  appl
  ops)

(defmethod SELECT-DESIGN-STATE (appl (method  HC-DESIGN-METHOD) space)
  appl method
  (setf space  (sort space #'(lambda (x y) 
                               (if (= (state-depth x) (state-depth y))
                                 (cheaper-state? x y)
                                 (> (state-depth x) (state-depth y))))))
  (when space
    (lines 2)
    (print-with-spaces 2 ;;; 2 ;;;  (state-depth (car space)) 
                       "Current space is ~s "
                       (if (> (length space)5)
                         (subseq space 0 5)
                         space)))
  (let ((s (loop for state in space
                 do
                 (when (and (state-feasible? state)
                            (not (violated-constraints state)))
                   (return state)))))
    (when s
      (lines 2)
      (print-with-spaces 2 
                         "Current state is now ~s"
                         s))
    
    s))


(defmethod EVALUATE-STATE-internal :after (appl (method  HC-DESIGN-METHOD) (state hc-design-state))
  (setf (state-cost state) (evaluate-state-cost appl method state)))

