;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-

(in-package "OCML")


(defvar *design-procedures*)
(defvar *design-fixes*)

(defmethod TOP-LEVEL-DESIGN (appl (method propose&revise))
  (setf  *design-procedures* 
         (mapcar #'(lambda (x)
                     (find-instance x 'design-procedure))
                 (the-slot-value (name method) 'has-design-procedures))
         *design-fixes* 
         (mapcar #'(lambda (x)
                     (find-instance x 'design-fix))
                 (the-slot-value (name method) 'has-design-fixes)))
  
  (cache-design-specification appl method)
  (clear-cached-links)
  (cache-param-to-ops-links)
  (cache-param-to-constraints-links)
  (cache-constraints-to-fixes-links)
  (construct-dependency-network)
  (unless (boundp '*dp-breaks*)
    (setf *dp-breaks* (breaks-in-dependency-network)))
  (initialise-design-space appl method)
  (top-level-design-loop appl method))


(defun clear-cached-links ()
  (loop for op in *design-procedures*
        do
        (setf ( effective-parameter-list op) nil))
  
  (loop for op in *design-fixes*
        do
        (setf (effective-constraint-list op) nil))
  
  (loop for p in *design-parameters*
        do
        (setf (relevant-operators p) nil)
        (setf (relevant-constraints p) nil))
  
   (loop for c in *design-constraints*
        do
        (setf (effective-parameter-list c) nil)
        (when (instance-of? (name c)
                            'fixable-parametric-constraint)
          (setf (relevant-operators c) nil))))


(defun cache-param-to-ops-links ()
  (loop for op in *design-procedures*
        for params = (compute-effective-applicability-set op)
        do
        (loop for param in params
              do
              (link-param-to-operator (find-instance param 'parameter)
                                      op))))

(defun cache-constraints-to-fixes-links ()
  (loop for op in *design-fixes*
        for constraints = (compute-effective-applicability-set op)
        do
        (loop for c in constraints
              do
              (link-constraint-to-fix (find-instance c 'constraint)
                                      op))))

(defmethod link-constraint-to-fix ((c constraint) (op design-fix))
    (with-slots (effective-constraint-list) op
      (pushnew c effective-constraint-list)
      (pushnew op  (relevant-operators c))))

(defmethod  compute-effective-applicability-set ((d design-fix))
  (the-slot-value (name d) 'applicable-to-constraints))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod DESIGN-FROM-STATE (appl (method propose&revise) (state design-state))
  (if (state-feasible? state)
    (cond( (violated-constraints state)
           (revise-design appl method state))
         (t
          ;;;;;(setf *trace-design-process?* nil)
          (if (state-complete? state)
            (design-succeeds state)
            (extend-design appl method state))))
    (state-fails state)))


(defmethod REVISE-DESIGN (appl (method design-method) (state design-state))
 ;;(break)
  (let ((*trace-design-process?* t))
  (cond ((new-state? state)
         (setf (state-foci state) (collect-fixable-constraints appl method state)
               (new-state? state) nil)
         (if (set-difference (violated-constraints state)
                             (state-foci state))
           (state-fails state)
           (design-from-context appl method state :revise)))
        (t
         (backtrack-from-operator appl method state (current-operator state)
                                  (current-focus state) :revise)))))


(defmethod COLLECT-FIXABLE-CONSTRAINTS (appl (method design-method) (state design-state))
  appl
  (with-slots (violated-constraints) state
    (let ((instances (all-instances 'fixable-parametric-constraint)))
      (filter violated-constraints 
              #'(lambda (x)
                  (and (member x instances) 
                       (relevant-operators x)))))))

;;;APPLY-DESIGN-OPERATOR - basic-design-fix - fixable-parametric-constraint
;;;The assignment which violates the constraint is retrieved and the 
;;;fix applied to the culpright parameter, its value, and current design model
(defmethod  APPLY-DESIGN-OPERATOR ((design-op basic-design-fix)
                                   (state design-state)  
                                   (focus fixable-parametric-constraint)
                                   (context (eql :revise)))
(let ((ass (right-value focus (bad-assignments state))))
     (apply-design-operator design-op state 
                            (find-instance (car ass) 'parameter)
                            context)))








 