;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: ocml;   -*-

(in-package "OCML")


;;;INSTANCES-META
(defclass instances-meta (standard-class)
  ((instances :initform (make-hash-table) :accessor class-instances)))

;;;GET-DIRECT-INSTANCES
(defmethod get-direct-instances ((class instances-meta))
  (with-slots (instances) class
    (map-over-hash-table #'(lambda (x y)
                                 (declare (ignore x))
                                 y)
                         instances)))

;;;GET-ALL-INSTANCES
(defmethod get-all-instances ((class instances-meta))
  (with-slots (instances) class
    (nconc (get-direct-instances class)
            (mapcan #'get-all-instances (subclasses nil class)))))

;;;ADD-INSTANCE-TO-CLASS
;;;Adds an instance to its class instances.  If it is a new one,
;;;the rule system is informed.
(defmethod add-instance-to-class ((class instances-meta)instance name)
  (with-slots (instances) class
    (setf (gethash name instances)instance)))



;(defmethod add-instance-to-class ((class instances-meta)instance name)
;  (with-slots (instances) class
;    (let ((already-exists? (gethash name instances)))
;      (setf (gethash name instances)instance)
;      (if already-exists?
;          ;;If it already exists it is going to be modified.  So, we better tell the
;          ;;rule system to remove all the associated info
;          (remove-all-slot-information-from-fc-rules instance)
;	  (tell-fc-rules (name class) (List name))))))

;;;REMOVE-INSTANCE-FROM-CLASS
(defmethod remove-instance-from-class ((class instances-meta)  name)
  (with-slots (instances) class
    (remhash name instances)))
  ;;;;;;;;(unassert-from-fc-rules (name class) (List name)))


(defmethod find-instance-in-class ((class instances-meta)name)
  (with-slots (instances) class
    (gethash name instances)))

;;;FIND-ALL-SLOT-MATCHES-IN-CLASS-INSTANCES INSTANCES-META
;;;Finds all the matches between the slot values of a particular slot of all direct
;;;instances of a class and a given set of args.
;;;NOT USED
(defmethod find-all-slot-matches-in-class-instances ((class instances-meta) slot args &optional env)
  (with-slots (instances) class
    (loop 
     for instance  being the hash-value in instances
     for name = (name instance)
     for values = (get-slot-values instance slot)
     append
     (loop with result2
	   with match
	   for value in values
	   do
	   (setf match (match (list name value) args env))
	   (unless (eq match :fail)
	     (push match result2))
	   finally
	   (return result2))
     into result
     finally
     (return (or result :fail)))))

;;;FIND-ONE-SLOT-MATCH-IN-CLASS-INSTANCES ---Like find-all-slot-matches-in-class-instances
;;;but it only returns the first match
(defmethod find-one-slot-match-in-class-instances ((class instances-meta) slot args &optional env)
  (with-slots (instances) class
    (loop with result = :fail
          for instance  being the hash-value in instances
          for name = (name instance)
          for values = (get-slot-values instance slot)
          do
          (setf  result (loop with match = :fail
                             for value in values
                             do
                             (setf match (match (list name value) args env))
			     while (eq match :fail)
                             finally
                             (return match)))
          while (eq result :fail)
          finally
          (return result))))

(defmethod remove-all-slot-values-from-direct-instances ((class instances-meta) slot no-checks)
   (with-slots (instances) class
     (loop for instance  being the hash-value in instances
           do
	   (remove-local-slot-values instance slot no-checks))))
 

(defmethod remove-slot-value-from-direct-instances ((class instances-meta) slot value no-checks)
   (with-slots (instances) class
     (loop for instance  being the hash-value in instances
           do
           (maybe-remove-slot-value instance slot value no-checks))))


;(defmethod remove-slot-value-from-all-direct-instances ((class instances-meta) slot value)
;   (with-slots (instances) class
;     (multiple-value-bind (inherited-values inherited-default-values)
;         (get-slot-values-from-class (name class)slot)
;       (loop for instance  being the hash-value in instances
;             for local-values = (slot-value instance slot)
;             when (member value local-values)
;             do
;             (maybe-remove-slot-value2 instance slot value
;                                     local-values
;                                     inherited-values
;                                     inherited-default-values)))))


;;;VALIDATE-SUPERCLASS --- This is a boring thing you have to do to ensure your metaclass
;;;definition is accepted
(eval-when (compile eval load)
  (defmethod clos::validate-superclass ((class instances-meta )(superclass standard-class))
    class superclass
    t))

;;;OCML-METACLASS 
(defclass ocml-metaclass (;;;;onto-spec-mixin
                          instances-meta)
  ((instance-var :initarg :instance-var)
   (ocml-name  :accessor name)
   (domain-slots :accessor domain-slots)            ;;;all domain slots (local + inherited)
   (local-slots :accessor local-slots :initform nil);;;all local slots
   (ocml-options :accessor ocml-options)
   (slot-info-alist :initform nil :accessor slot-info-alist)))

(defmethod internal-name ((class ocml-metaclass))
  (class-name class))

(defmethod direct-domain-superclasses ((class ocml-metaclass))
  (remove (find-class 'basic-domain-class)(direct-superclasses nil class)))

(defmethod domain-superclasses ((class ocml-metaclass))
  (domain-superclasses-recursively class nil nil))

(defun domain-superclasses-recursively (current pool result)
  (let ((new (set-difference (direct-domain-superclasses current)
                             (union pool result))))
    (setf pool (nconc new pool))
    (if pool
      (domain-superclasses-recursively (car pool) (cdr pool) (cons (car pool) result))
      result)))


(defmethod inherited-slots ((class ocml-metaclass))
  (with-slots (domain-slots local-slots)
      class
  (set-difference domain-slots local-slots)))

;;;SET-METACLASS-INFORMATION
(defmethod set-metaclass-information ((class ocml-metaclass)
                                      domain-slots1        ;;;all domain slots (Local + inherited)
                                      ocml-slots-info      ;;;all ocml slot info (local + inherited)
                                      &optional
                                      name
                                      local-slots1         ;;;all local slots
                                      instance-var1  
				      ocml-options1        ;;;local ocml options
                                      )
  (with-slots (ocml-name domain-slots local-slots instance-var ;;;;onto-spec
                            ocml-options slot-info-alist)
      class
    (setf  domain-slots domain-slots1 slot-info-alist ocml-slots-info)
    (when name
      (setf local-slots local-slots1
            ocml-name name 
	    instance-var instance-var1
            ;;;;            onto-spec ontolingua-spec
            ocml-options ocml-options1
            ))))

;;;CLASS-INHERITED-SLOTS ---Returns all the slots inherited from a list of (super-)classes
(defun class-inherited-slots (superclasses)
  (remove-duplicates 
   (loop for class in superclasses
	 appending (domain-slots class))))


;;;FIND-OPTION-VALUE ---
(defmethod find-option-value ((class ocml-metaclass)slot option)
  (with-slots (slot-info-alist) class
    (let ((structure (right-value slot slot-info-alist)))
      (when structure
        (get-slot-info-from-structure option structure)))))
         
         

;;;VALIDATE-SUPERCLASS --- This is a boring thing you have to do to ensure your metaclass
;;;definition is accepted
(eval-when (compile eval load)
  (defmethod clos::validate-superclass ((class ocml-metaclass )(superclass standard-class))
    class superclass
    t))

;;;ADD-SLOT-OF-LINKS
(defmethod add-slot-of-links ((class ocml-metaclass))
  (with-slots (domain-slots)class
    (loop for slot in domain-slots
 	  do
          (add-slot-of-entry (find-or-create-relation slot 2) class))))



;;;ENSURE-CLASS-IS-RELATION ---This function is called to ensure that when a class
;;;is created, the associated unary relation is created as well, if it does not exist
;;;already.  
(defun ensure-class-is-relation (class instance-var relation-spec)
  (let ((rel (get-relation class)))
    (if rel
        ;;if rel exists, then we simply add the relevant information
        (when relation-spec
	  (apply #'add-relation-spec rel instance-var relation-spec))
        ;;otherwise we create the new relation.
        (apply #'make-ocml-relation  class
               :arity 1
	       :schema (if instance-var
                           (list instance-var)
                           (list (make-new-var)))
	       relation-spec))))


;;;MATCH-ONE-CLASS-INSTANCE
;(defmethod match-one-class-instance ((class ocml-metaclass)arg more-args &optional env try-subclasses?
;                                     &aux (result :fail))
;  (with-slots (instances) class
;    (Let ((new-env (loop 
;                    for instance being the hash-value in instances
;                    do
;                    (setf result
;                          (match-spec-against-instance
;                           instance arg more-args env))
;		    while (eq result :fail)
;                    finally
;                    (return result))))
;      (cond ((eq new-env :fail)
;	     (loop for subclass in (direct-subclasses nil class)
;		   do
;		   (Setf new-env (match-one-class-instance subclass arg more-args env t))
;		   while (eq new-env :fail)
;		   finally (return new-env)))
;            (t
;             new-env)))))


;;;MATCH-ALL-CLASS-INSTANCES
;(defmethod match-all-class-instances ((class ocml-metaclass) arg more-args
;                                      &optional env try-subclasses?
;                                      &aux result)
;  (with-slots (instances) class
;    (loop
;     with envs = :fail
;     for instance being the hash-value in instances
;     do
;     (setf envs (match-spec-against-instance instance arg more-args env t))
;     (unless (eq envs :fail)
;       (setf result (append result envs))))
;    (when try-subclasses?
;	(loop for subclass in (direct-subclasses (class-name class))
;              for result2 = (match-all-class-instances subclass arg more-args env try-subclasses?)
;              do
;              (unless (eq result2 :fail)
;                (setf result (append result result2)))))
;    (or result :fail)))


(defmethod get-min-cardinality ((class t) slot)
  (find-option-value (get-domain-class class) slot :min-cardinality))

(defmethod get-max-cardinality ((class t) slot)
  (find-option-value (get-domain-class class) slot :max-cardinality))

(defmethod get-min-cardinality ((class ocml-metaclass) slot)
  (find-option-value class slot :min-cardinality))

(defmethod get-max-cardinality ((class ocml-metaclass) slot)
  (find-option-value class slot :max-cardinality))

;;;GET-SLOT-VALUES-FROM-CLASS
(defun get-slot-values-from-class (name slot &optional no-defaults)
  (let ((class (get-domain-class name)))
    (if class
        (get-slot-values-from-class-structure class slot no-defaults)
	(error "~S is not a class" name))))

;;;GET-SLOT-VALUES-FROM-CLASS-STRUCTURE
(defmethod get-slot-values-from-class-structure ((class ocml-metaclass) slot no-defaults)
  (values
   (find-option-value class slot :value)
   (unless no-defaults
     (find-option-value class slot :default-value))))

;;;GET-DEFAULT-SLOT-VALUES-FROM-CLASS-STRUCTURE
;(defmethod get-default-slot-values-from-class-structure ((class ocml-metaclass) slot)
;  (find-option-value class slot :default-value))

;;;GET-DEFAULT-SLOT-VALUES-FROM-CLASS
(defun get-default-slot-values-from-class (class slot)
  (find-option-value class slot :default-value))
  

;;;*************************************************************************************

(defvar *domain-classes* (make-hash-table))

(defun generate-internal-ocml-class-name (name)
  (gentemp (string name)))

;;;GET-OCML-CLASS
(defun get-ocml-class (Class)
  (gethash class *domain-classes*))
  ;(and (gethash class *domain-classes*)
;       (find-class class)))

;;;GET-DOMAIN-CLASS ---The same as get-ocml-class.  This function will
;;;disappear once I make sure it is not used anywhere.
(defun get-domain-class (class)
  (gethash class *domain-classes*))
  ;(and (gethash class *domain-classes*)
;       (find-class class)))

(defun domain-class? (class-structure)
  (typep class-structure 'ocml-metaclass))

  

(defun remove-domain-class (class)
  (when (get-domain-class class)
    (remhash class *domain-classes*)
    ;;;;;(remove-class class)
    ))

(defun add-domain-class (name class)
  (setf (gethash name *domain-classes*)
        class))

;(defun add-domain-class (class)
;  (setf (gethash class *domain-classes*)
;        t))

(defun remove-all-classes ()
  ;(maphash #'(lambda (class ignore)
;               ignore
;               (remove-class class))
;           *domain-classes*)
  (clrhash  *domain-classes*))

;;;BASIC-DOMAIN-CLASS
(defclass basic-domain-class (name-mixin  basic-ocml-object)
  ())

  
;;;INITIALIZE-INSTANCE :AFTER BASIC-DOMAIN-CLASS
(defmethod initialize-instance :after ((instance basic-domain-class) &rest initargs)
  (with-slots (name) instance
    (add-instance-to-class (find-class (type-of instance)) instance name)))

(defmethod print-object ((instance basic-domain-class) stream)
  (with-slots (name)instance
    (format stream "#<~S ~S>" (name (find-class (type-of instance))) name)))



;;;DEFINE-DOMAIN-CLASS --- This is the entry point for class definition
(defun define-domain-class (name superclasses instance-var  documentation
                                       class-slots relation-spec)
  (multiple-value-bind (instance-var1  documentation1 class-slots1 relation-spec1)
      (parse-class-form instance-var  documentation class-slots relation-spec)
    (do-class-definition name superclasses instance-var1  documentation1
				class-slots1 relation-spec1)))
    ;(when redefinition?
;      (redefine-subclasses class))))



;;;CREATE-INITIAL-CLASS-DEFINITION ---This is called to create an initial definition of the class.
;;;The reason why I need to do this is that I need to have the class in place before I can
;;;call finalize-class-spec.
(defun create-initial-class-definition (name superclasses slots)
  (setf superclasses (mapcar #'(lambda (super)
                                 (internal-name (get-ocml-class super)))
                             superclasses))
  (eval `(defclass ,name ,superclasses ,slots (:metaclass ocml-metaclass))))

;;;DO-CLASS-DEFINITION
(defun do-class-definition (name superclasses instance-var  documentation
				 class-slots relation-spec
                                 &aux lost-slots new-class slots-to-recompute domain-slots)  
  (Let* ((slots-spec (parse-class-slots name class-slots))
         (local-slots (mapcar #'car slots-spec))
	 (supers (mapcar #'get-domain-class superclasses))
         (inherited-slots (set-difference (class-inherited-slots supers)
                                          local-slots))
         (class (get-domain-class name))
	 (already-exists? class)
         (internal-name (if already-exists?
                            (internal-name class)
                            (generate-internal-ocml-class-name name))))
    (when already-exists?
      (setf lost-slots (set-difference (domain-slots class) (append local-slots inherited-slots))))
    (setf class (create-initial-class-definition internal-name superclasses local-slots))
    (multiple-value-bind (clos-specs slot-info-list ocml-options)
        (finalize-class-spec  supers
                              (filter (cdr (class-precedence-list class))
				      #'domain-class?)
                              slots-spec
                              inherited-slots)
      ;;;;(append (mapcar #'list inherited-slots)
      ;;;; slots-spec))
      (setf new-class (make-domain-class name internal-name supers  instance-var documentation
			                 clos-specs
                                         (append local-slots inherited-slots)
                                         local-slots
                                         ocml-options
                                         slot-info-list
			                 relation-spec))
      (when already-exists?
        (dolist (slot lost-slots)
          (remove-slot-of-entry (get-relation slot)class))
        (Setf domain-slots (domain-slots new-class)
              slots-to-recompute                   ;;The slots to recompute are all the domain slots
              (union (domain-slots new-class)      ;;of the new class, + the slots 
		     lost-slots))                  ;;which have been 'lost'
       	(update-class-direct-instances new-class
                                       domain-slots);;Only domain slots are needed for instances 
	(update-subclasses new-class slots-to-recompute  lost-slots)) 
      new-class)))



;;;MAKE-DOMAIN-CLASS
(defun make-domain-class (name internal-name supers  instance-var  documentation
                               slots-spec       ;;clos-like spec
                               domain-slots     ;;All domain slots (local and inherited)
                               local-slots
                               ocml-options
                               ocml-slot-info   ;;local + inherited ocml options
			       relation-spec
                               ;;;;redefinition?
                               &aux class)
  (prog1
      (setf class
	    (eval `(defclass ,internal-name ,(ensure-vanilla-class2
		                              supers 
		                              'basic-domain-class)
           , slots-spec
           ,(list :documentation documentation)
           ,(list :metaclass 'ocml-metaclass))))
    (add-domain-class name class)
    (set-metaclass-information class domain-slots ocml-slot-info
			       name local-slots  instance-var  ;;;;;ontolingua-spec
                                ocml-options)
    (ensure-class-is-relation name instance-var relation-spec)
    (add-slot-of-links class)))


;;;COMPUTE-SLOTS-TO-RECOMPUTE
;;;This function returns the list of slots which need to be recomputed after
;;;a class redefinition.  This contains the old and the new local slots of the class,
;;;+ all the slots inherited from old superclasses (which are not supers anymore)
;;;and those inherited from new supers (which were not supers before).
;;;NOT USED
;(defun compute-slots-to-recompute (class new-supers new-local-slots)
;  (union (local-slots class)
;         (union new-local-slots
;                (class-inherited-slots
;                 (set-exclusive-or new-supers
;                                   (direct-domain-superclasses  class))))))

(defun update-subclasses (class slots-to-recompute lost-slots);;;; new-supers lost-supers)
  (loop 
        with subclasses = (remove-duplicates         ;;;Get all subclasses (including the indirect ones)
                           (subclasses nil class))
	while subclasses
        for subclass = (pick-next-subclass subclasses)
        do
	(redefine-subclass-after-class-update subclass slots-to-recompute lost-slots) 
	(setf subclasses (remove subclass subclasses))))

(defun pick-next-subclass (subclasses)
  (find nil subclasses :test #'(lambda (x y)
                                 (declare (ignore x))
                                 (null (intersection
                                        (direct-superclasses nil y)
                                        subclasses)))))
        
;;;REDEFINE-SUBCLASS-AFTER-CLASS-UPDATE
(defmethod redefine-subclass-after-class-update ((class ocml-metaclass)
                                                 slots-to-recompute
                                                  lost-slots) ;;;; new-supers lost-supers) 
  (with-slots (local-slots ocml-options) class
    (let*  ((relevant-local-slots (intersection local-slots slots-to-recompute))
	    (superclasses (direct-superclasses nil class))
            (domain-slots (class-inherited-slots superclasses))
            (other-relevant-slots (intersection
                                   (set-difference domain-slots relevant-local-slots)
                                   slots-to-recompute))
            (slot-info-list)
	    (class-precedence-list (class-precedence-list class)))
      (loop for slot in relevant-local-slots
            for slot-info = (calculate-slot-info 
			     superclasses
                             class-precedence-list
	                     slot
                             (right-value slot ocml-options))
	    do
	    (push (cons slot slot-info) slot-info-list))
      (loop for slot in other-relevant-slots
            for slot-info = (calculate-slot-info 
	                     superclasses
                             class-precedence-list
	                     slot
                             nil)           ;;;Of course there are no local ocml options
            do
	    (push (cons slot slot-info) slot-info-list))
      (set-metaclass-information class
                                 (mapcar #'car slot-info-list) ;;domain slots
                                 slot-info-list)
      (when lost-slots
        (dolist (slot (set-difference lost-slots (domain-slots class)))
          (remove-slot-of-entry (get-relation slot)class)))
      (update-class-direct-instances class (append relevant-local-slots other-relevant-slots)))))



;(defmethod redefine-subclass-after-class-update ((class ocml-metaclass)
;                                                 slots-to-recompute
;                                                 lost-slots) ;;;; new-supers lost-supers) 
;  (with-slots (local-slots ocml-options) class
;    (let* ((superclasses (direct-superclasses nil class))
;           (domain-slots (class-inherited-slots superclasses))
;           (relevant-domain-slots (intersection slots-to-recompute domain-slots))
;           (relevant-inherited-slots
;            (set-difference relevant-domain-slots local-slots))
;           (relevant-local-slots (intersection local-slots slots-to-recompute))
;	   (class-precedence-list (class-precedence-list class))
;           (slot-info-list))
;      (loop for slot in relevant-local-slots
;            for slot-info = (calculate-slot-info 
;			     superclasses
;                             class-precedence-list
;	                     slot
;                             (right-value slot ocml-options))
;	    do
;	    (push (cons slot slot-info) slot-info-list))
;      (loop for slot in relevant-inherited-slots
;            for slot-info = (calculate-slot-info 
;	                     superclasses
;                             class-precedence-list
;	                     slot
;                             nil)           ;;;Of course there are no local ocml options
;            do
;	    (push (cons slot slot-info) slot-info-list))
;      (set-metaclass-information class
;                                 (mapcar #'car slot-info-list) ;;domain slots
;                                 slot-info-list)
;      (when lost-slots
;        (dolist (slot (set-difference lost-slots (domain-slots class)))
;          (remove-slot-of-entry (get-relation slot)class)))
;      (update-class-direct-instances class domain-slots))))

    
(defmethod update-class-direct-instances ((class ocml-metaclass)slots-to-recompute)
                                             ;;;;;new-supers lost-supers)
  (with-slots (instances) class
    (loop for instance being the hash-value in instances
          do
          ;(update-fc-instance-of-links             ;;;Deleted, as we now only link objects to fc rules
;           (name instance) new-supers lost-supers) ;;;at compile-ruleset time
          (update-instance-after-class-redefinition instance class slots-to-recompute))))

         

;;;FINALIZE-CLASS-SPEC --- It does the following:
;;;1. It computes the initform of the class.  This
;;;will comprise the value of the associated slot.  This value might be
;;;inherited from superclasses
;;;2. It creates a slot-info structure which includes all the ocml options
;;;either local or  inherited from superclasses
;;;3. It records all local ocml options associated with each slot
;;;
(defun finalize-class-spec (superclasses class-precedence-list local-slot-specs inherited-slots)
  (loop with result = nil
        with slot-info-list = nil
        with ocml-options-record = nil
	for slot-spec in local-slot-specs
        ;;each slot-spec is a list of 1 to 3 elements
        do
	(destructuring-bind (slot &optional ocml-options clos-options)
	    slot-spec
          (let ((slot-info 
                 (calculate-slot-info 
	          superclasses
	          class-precedence-list
	          slot ocml-options)))
            (push 
	     `(,slot
	       :initarg ,slot 
	       ,@(apply #'append clos-options))
             result)
            (push (cons slot slot-info) slot-info-list)
            (push (cons slot ocml-options)ocml-options-record)))
        finally
        (Loop for slot in inherited-slots
             for slot-info = (calculate-slot-info 
	                      superclasses
	                      class-precedence-list
	                      slot nil)
             do
             (push (cons slot slot-info) slot-info-list))
        (return (values result slot-info-list ocml-options-record))))
 


;;;STRUCTURE SLOT-INFO
(defstruct (slot-info (:conc-name ocml-slot-))
  (min-cardinality)
  (max-cardinality)
  (value)
  (default-value)
  (inheritance)
  (type))

(defun get-slot-info-from-structure (option structure)
  (funcall (read-from-string (string-append "OCML-SLOT-" (string option)))
           structure))
   
(defun calculate-slot-info (superclasses precedence-list slot ocml-options)
  (let* ((inheritance (decide-inheritance-type precedence-list slot ocml-options))
        (values (decide-value-options  superclasses  precedence-list slot ocml-options
                                       inheritance))
        (type (decide-type-option  superclasses  slot ocml-options ))
        (min-cardinality  (decide-min-cardinality
                           superclasses  slot ocml-options ))
        (max-cardinality  (decide-max-cardinality
                           superclasses  slot ocml-options )))
    (create-slot-info-structure
     (List :inheritance inheritance
		     :value (car values)
		     :default-value (second values)
                     :min-cardinality min-cardinality
                     :max-cardinality max-cardinality
                     :type type))))
           
		    ;(append
;		     (list :inheritance inheritance)
;		     (list :value (car values))
;		     (list :default-value (second values))
;                     (list :min-cardinality min-cardinality)
;                     (list :max-cardinality max-cardinality)
;                     (list :type type)))))


(defun create-slot-info-structure (list)
  (apply #'make-slot-info list))

(defun decide-type-option ( superclasses  slot ocml-options )
   (remove-duplicates (find-all-option-values superclasses slot :type ocml-options)
                      :test #'equal))

(defun decide-min-cardinality (superclasses  slot ocml-options)
  (let ((cardinalities (find-all-option-values
                        superclasses slot :min-cardinality ocml-options t)))
    (when cardinalities
      (apply #'min cardinalities))))

(defun decide-max-cardinality (superclasses  slot ocml-options )
  (let ((cardinalities (find-all-option-values
                        superclasses slot :max-cardinality ocml-options t)))
    (when cardinalities
      (apply #'max cardinalities))))


(defun decide-inheritance-type (precedence-list slot ocml-options)
  (multiple-value-bind (value flag)
      (find-first-option-value precedence-list slot :inheritance ocml-options )
    (if value
        (if flag
            (car value)
            value)
         *default-inheritance*)))

(defun find-first-option-value (precedence-list  slot option ocml-options)
  (let ((pair (assoc option ocml-options)))
    (if pair
        (values (cdr pair) :list)
	(loop for class in precedence-list
            for result = (find-option-value class slot option)
            until result
            finally (return result)))))


(defun find-all-option-values (superclasses slot option ocml-options &optional collect?)
  (let ((pair (assoc option ocml-options)))
    (if pair
        (cons (second pair)(collect-all-option-values superclasses slot option collect?))
        (collect-all-option-values superclasses slot option collect?))))

(defun collect-all-option-values (superclasses slot option &optional collect?)
  (if collect?
       (loop
        with result = nil
        for class in superclasses
	for option-value = (find-option-value class slot option)
        when option-value
        do
        (setf result (cons option-value result))
        finally
        (return result))
       (loop
        for class in superclasses
	for option-value = (find-option-value class slot option)
	appending option-value)))

;;;DECIDE-VALUE-OPTIONS  ---Returns a list of two elements.  The first contains
;;;all definitional slot values for this slot, and the second all the relevant slot
;;;values
(defun decide-value-options (superclasses precedence-list slot ocml-options inheritance)
  (list
   (remove-duplicates (find-all-option-values superclasses slot :value ocml-options)
                      :test #'equal)
   (remove-duplicates
    (case inheritance
      (:supersede
       (find-first-option-value precedence-list slot :default-value ocml-options))
      (:merge
       (find-all-option-values superclasses slot :default-value ocml-options)))
    :test #'equal)))



;(defun generate-slot-info-label (slot)
;  (read-from-string (string-append (string slot)
;                                   "-INFO$$$")))


