;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: ocml;   -*-

(in-package "OCML")

(defun compile-function (name)
  (Let ((function (get-function name))
        )
    (unless function
      (error "~Can't compile function ~S, which has not been defined" name))
    (with-slots (lisp-fun body schema) function
      (cond (lisp-fun
             (ocml-output "No need to compile ~S, which has already a lisp attachment"name))
            (body
            ;;;(setf lisp-fun
              ;;     (compile-attachment
                    (compile-function-body  body schema))
		 ;;;    ))
            (t
             (error "Can't compile function ~s, whose body has not been specified"
                    name))))))

(defun compile-function-body (body schema)
  `(function (lambda ,schema
               ,(compile-term body schema))))


;;;COMPILE-FUNCTION-CALL
(defun compile-function-call (fun args schema)
  (case fun
     (quote (list 'quote (car args)))
     (if (compile-if-form args schema))
     (cond (compile-cond-form args schema))
     (in-environment `(evaluate-in-env-form ,args nil))
     (findall `(evaluate-findall-form ,args nil))
     (the `(evaluate-the-form ,args nil))
     (tell `(evaluate-tell-form ,(first args)nil))
     (call (compile-call-form args schema))
     (rcall `(evaluate-rcall-form ,args nil))
     (list-of `(list ,@(mapcar #'(lambda (arg)
                                 (compile-term arg schema))
                             args)))
     (lambda `(cons lambda ,args))              ;If it is a lambda we just return it
     (t
      (let ((fun-structure (get-function fun)))
	(if fun-structure
            `(funcall (lisp-fun (get-function ',fun))
                      ,@(mapcar #'(lambda (x)(compile-term x schema)) args))
            (error "Cannot compile ~S which is not an OCML function" fun))))))

(defun compile-call-form (args schema)
  `(evaluate-call-form
         (list ,@(mapcar #'(lambda (arg)(compile-term arg schema))
                                           args))
         nil))


;;;This is always called within an OCML-EVAL
(defun compile-term (term schema)
  (cond  ((consp term)
          (compile-function-call (car term)(cdr term) schema))
         ((variable? term)
	  (if (member term schema)
              term
              (values (list 'quote term) t)))
         (t
          term)))


(defun compile-if-form (args schema &optional env)
  (Let ((a (gentemp "RESULT")))
    `(let ((,a (ask-top-level ,(compile-goal-expression (car args) schema))))
       (if (eq ,a :fail)
           ,(if (cddr args)
                (compile-term (third args)schema)
                *nothing*)
           ,(multiple-value-bind (exp flag)
               (compile-term (second args) schema)
             (if flag
                 `(ocml-eval ,exp ,a)
                 exp))))))

;(defun compile-if-form (args schema)
;  (Let ((a (gentemp "RESULT")))
;    `(let ((,a (ask-top-level ,(compile-goal-expression (car args) schema))))
;       (if (eq ,a :fail)
;           ,(if (cddr args)
;                `(ocml-eval ,(compile-term (third args)schema))
;                *nothing*)
;           ,`(ocml-eval ,(compile-term (second args) schema) ,a)))))

;(defun compile-term (term schema)
;  (cond  ((consp term)
;          (compile-function-call (car term)(cdr term) schema))
;         ((variable? term)
;	  (if (member term schema)
;              `(list 'quote ,term)
;	      (list 'quote term)))
;         (t
;          (list 'quote term))))

(defun  compile-cond-form (args schema)
  (Let ((a (gentemp "RESULT")))
    `(let (,a)
       ,(cons 'cond (nconc
                     (loop for clause in args
                      for goal = (car clause)
                      for term = (second clause)
                      collecting (compile-cond-clause a goal term schema))
                     `((t ,*nothing*)))))))

(defun compile-cond-clause (a goal term schema)
  `((progn
      (setf ,a (ask-top-level ,(compile-goal-expression goal schema)))
      (not (eq ,a :fail)))
    (ocml-eval ',term ,a)))

(defun compile-goal-expression (goal schema)
  (cond ((member (car goal) '(and or))
         `(cons ',(car goal)
                (list
                 ,@(mapcar #'(lambda (x)(compile-goal-expression x schema))
                       (cdr goal)))))
        ((eq (car goal) 'not)
         `(list not ,(compile-goal-expression (second goal) schema)))
       (t
        ;;goal is a basic relational expression
        `(cons ',(car goal)
               (list 
                ,@(mapcar #'(lambda (x)
                          (compile-expression-argument x schema))
                      (cdr goal)))))))

(defun compile-expression-argument (arg schema)
  (if (atom arg)
      (if (and (variable? arg)(member arg schema))
          arg
          (list 'quote arg))
      `(cons ,(compile-expression-argument (car arg) schema)
            ,(compile-expression-argument (cdr arg) schema))))  

;(def-function partition-params (?params ?bound-ones ?unbound-ones)
;   :body (if (null? ?params)
;             (list-of ?bound-ones ?unbound-ones)
;             (if (has-value #e(first ?params) ?value)
;                 (partition-params (rest ?params)
;                                   (cons (first ?params)
;                                         ?bound-ones)
;                                   ?unbound-ones)
;                 (partition-params (rest ?params)
;                                    ?bound-ones
;                                   (cons (first ?params)
;					 ?unbound-ones))))
;   :Lisp-fun #'(lambda (params b u)
;                 (partition-params params b u)))
