;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: ocml;   -*-


;;;This stuff has been modified by mauro

(in-package "OCML")



(defun compile-f-or-p (function &optional verbose force-p)
  (if (procedure-p function)
      (compile-procedure function verbose force-p)
      (compile-function function verbose force-p)))


(defun compile-procedure (procedure &optional verbose force-p)
  (compile-function procedure verbose force-p "procedure" #'compile-procedure-body))
  

(defun compile-function (function &optional verbose force-p (type "function")
                                  (fun #'compile-function-body))
  (with-slots (lisp-fun body schema name user-defined? free-vars-in-body) function
    (unless user-defined?           ;;It is defined by a lisp fun
           ;(when verbose
;                 (ocml-output "~%No need to compile ~a ~S" type name)))
      (cond
       ((or force-p (not lisp-fun))
	(cond (body
	       (when verbose
		 (ocml-output "~%Compiling ~a ~S"type name))
	       (setf lisp-fun
		     (compile-attachment
		      (funcall fun  body schema free-vars-in-body)
		      name)))
	      (t
	       (error "Can't compile ~a ~s, whose body has not been specified"
		      type
		      name))))
       (lisp-fun
	(when verbose
	  (ocml-output "No need to recompile ~a ~S" type name)))))))

  


(defun compile-function-body (body schema &optional free-vars-in-body)
  `(function (lambda ,(cons 'env schema)
               (let ,(handle-free-vars-in-body free-vars-in-body)
               ,(compile-term body
                              (mapcar #'cons schema schema) ;; to be compatible with back
                              'env)))))

(defun handle-free-vars-in-body (vars)
  (mapcar #'(lambda (var)
              (list var (List 'gensym (string var))))
          vars))

(defun compile-procedure-body (body schema &optional free-vars-in-body)
  `(function (lambda ,(cons 'env schema)
               (let ,(handle-free-vars-in-body free-vars-in-body)
               ,(compile-control-term body
                                      (mapcar #'cons schema schema) ;; to be compatible with back
                                      'env)))))

(defun compile-function-call (fun args schema env )
  (if *compiled*
      (compile-function-call-cmp fun args schema env )
    (compile-function-call-int fun args schema env )))

(defun compile-function-call-int (fun args schema env )
  
  (case fun
    (quote (list 'quote (car args)))
    (if (compile-if-form args schema env))
    (cond (compile-cond-form args schema env))
    (in-environment (compile-in-env-form args schema env))
    (findall `(findall ,(if (consp (car args))
                          `(list ,@(mapcar #'identity (car args)))
                          (car args))
                       ,(compile-goal-expression (second args)schema env )
                       ,env))
    (setofall `(setofall ,(if (consp (car args))
                          `(list ,@(mapcar #'identity (car args)))
                          (car args))
                       ,(compile-goal-expression (second args)schema env )
                       ,env))
    (the `(findany ,(if (consp (car args))
                      `(list ,@(mapcar #'identity (car args)))
                      (car args))
                   ,(compile-goal-expression (second args)schema env )
                   ,env))
    (eval `(ocml-eval-gen  ,(compile-term (car args) schema env) ,env) )
    (apply `(ocml-apply-internal  ,(compile-term (car args) schema env) 
                                  ,(compile-term (second args) schema env)))
    (call `(ocml-apply-internal  ,(compile-term (car args) schema env) 
                                  (list ,@(mapcar #'(lambda (x)
                                               (compile-term x schema env))
                                           (cdr args)))))
    (t
     (compile-defined-function fun args schema env))))   

(defun compile-function-call-cmp (fun args schema env )
  
  (case fun
    (quote (list 'quote (car args)))
    (if (compile-if-form args schema env))
    (cond (compile-cond-form args schema env))
    (in-environment (compile-in-env-form args schema env))
    (findall 
      (let ((nenv (gensym "env")))
	`(let (res envs ,nenv)
	    (declare (special res envs))
	    ,@(compile-body (list (second args)) schema 
				  #'(lambda (ienv)
				      (setf res t)
				      (push ienv envs)
				      nil)
			    nil env)
	    (when res 
	      (mapcar #'(lambda (ienv)
			  (instantiate ,(compile-back-term
					 (first args) 
					 schema
					 env)
				       ienv))
		      envs)))))
    (setofall 
     ;;;;(let ((nenv (gensym "env")))
       `(let (res envs);;;; ,nenv)
	  (declare (special res envs))
	  ,@(compile-body (list (second args)) schema 
			  #'(lambda (ienv)
			      (setf res t)
			      (push ienv envs)
			      nil)
			  nil env)
	  (when res 
	      (remove-duplicates (mapcar #'(lambda (ienv)
			                     (instantiate ,(compile-back-term
					                    (first args) 
					                    schema
					                    env)
				                          ienv))
		                         envs)))))
    (the 
     `(let (ienv)
	(setf ienv 
	      ,@(compile-body (list (second args)) schema #'true-cont
			      nil env))
	(if ienv
	  (instantiate ,(compile-back-term
			 (first args) 
			 schema
			 env)
		       ienv)
          :nothing)))
    (t
     (compile-defined-function fun args schema env))))

(defun compile-procedure-call (fun args schema env )
  (case fun
    
    (do-actions
     `(prog ()
        ,@(mapcar #'(lambda (x)(compile-control-term x schema env ))
                  args)))
    (repeat-actions (multiple-value-bind (actions test flag)
	                (parse-repeat-construct args)
                      (compile-repeat-actions-form schema actions test flag env )))
    (loop (compile-loop-form args schema env ))
    (in-environment (compile-in-env-form args schema env t ))
    (if (compile-if-form args schema env t ))
    (cond (compile-cond-form args schema env t ))
    (tell
     `(tell1 ,(compile-tell-form (car args) schema env) nil ,env))
    (unassert `(unassert1 ,(compile-unassert-form (car args) schema env) 
                          ,env))
    (t
     (compile-function-call fun args schema env))))



(defun compile-unassert-form (exp schema env)
  (if (variable? exp)
      exp
      (compile-goal-expression exp schema env)))

(defun compile-tell-form (exp schema env)
  (compile-unassert-form exp schema env))


(defun compile-loop-form (args schema env )
  (destructuring-bind (for var in exp do &rest body)args
    (cond ((and (eq for 'for)
                (eq in 'in)
                (eq do 'do-actions))
           (compile-loop-form2 var exp body schema env ))
          (t
           (error "Bad syntax for Loop expression..when evaluating ~S"
                  (cons 'loop args))))))

(defun compile-loop-form2 (var exp body schema env )
  ;;;;(let (
        ;;;(var (gensym (string var))))
    `(loop for el in ,(compile-term exp schema env)
         do
         (setf ,env (bind ,var el env))
         ,@(mapcar #'(lambda (a)
		     (compile-control-term a schema env ))
		 body)))

     
(defun compile-repeat-actions-form (schema actions test flag env )
  (if flag
      `(loop 
        until (eq :fail ,(if *compiled* 
                             (compile-query test schema env)
                           `(ask-top-level ,(compile-goal-expression test schema env) 
                                          :env ,env)))
        do 
        ,@(mapcar #'(lambda (a)
		     (compile-control-term a schema env))
		 actions))
      `(loop 
	do
       ,@ (mapcar #'(lambda (a)
		     (compile-control-term a schema env ))
		 actions)
        until (eq :fail 
                  ,(if *compiled*
                      (compile-query test schema env)
                    `(ask-top-level ,(compile-goal-expression test schema env) 
                                    :env ,env))))))

(defun compile-lambda-args (args schema env )
  (if (consp args)
      `(list
       ,@(mapcar #'(lambda (x)
		     (compile-expression-argument x schema env))
		 args))
      (compile-expression-argument
	args schema env)))


(defun compile-defined-function (fun args schema env)
  (let ((fun-structure (get-function fun)))
    (if fun-structure
      (let ((compiled-args 
             (mapcar #'(lambda (x)
                         (compile-term x schema env ))
                     args)))
        (unless (user-defined? fun-structure)
          (push env compiled-args))
        `(funcall (lisp-fun (get-function ',fun))
                  ,@compiled-args))
      (error "Cannot compile ~S which is not an OCML function" fun))))




;;(defun compile-replace-form (exp1 exp2 schema env)
;;  `(replace1             
;;	 ,(compile-term exp1 schema env)
;;          ,(compile-term exp2 schema env)))


         

(defun call-or-funcall-1 (env fun &rest args)
  (if (functionp fun)
    (apply fun args)
    (if (lambda-exp? fun)
      (apply-lambda-exp fun args env) 
      (ocml-eval-fun-term fun args env))))

(defun compile-call-form (args schema env )
  `(call-or-funcall ,env ,@(mapcar #'(lambda (arg)
                                       (compile-term arg schema env))
		                   args)))

(defun call-or-funcall (env fun &rest args)
  (if (functionp fun)
    (apply fun args)
    (if (lambda-exp? fun)
      (apply-lambda-exp fun (mapcar #'(lambda (arg) (list 'quote arg))args) env) ;;;tried 4/1/96
      (ocml-eval-fun-term fun (mapcar #'(lambda (arg) (list 'quote arg))args) env))))




(defun compile-if-form (args schema  env &optional procedure?  )
  (Let ((a (gensym "RESULT"))
        (fun (if procedure?
                #'compile-procedure-call
                #'compile-function-call)))
    `(let ((,a ,(if *compiled*
                   (compile-query (car args) schema env)
                  `(ask-top-level
                   ,(compile-goal-expression (car args) schema env)
                   :env
                   ,env))))
       (if (eq ,a :fail)
           ,(if (cddr args)
                (compile-term (third args)schema env fun  )
                *nothing*)
           ,(compile-term (second args) schema a fun  )))))

(defun  compile-cond-form (args schema env &optional procedure?  )
  (Let ((a (gensym "RESULT"))
        (fun (if procedure?
                #'compile-procedure-call
                #'compile-function-call)))
    `(let (,a)
       ,(cons 'cond (nconc
                     (loop for clause in args
                      for goal = (car clause)
                      for term = (second clause)
                      collecting (compile-cond-clause a goal term schema env fun  ))
                     `((t ,*nothing*)))))))

(defun compile-cond-clause (a goal term schema env fun )
  `((progn
      ,(if *compiled*
           `(setf ,a ,(compile-query goal schema env))
         `(setf ,a (ask-top-level
                   ,(compile-goal-expression goal schema env )
                   :env
                   ,env)))
      (not (eq ,a :fail)))
    ,(compile-term term schema a fun  )))

(defun compile-in-env-form (args schema env &optional procedure?   &aux flag body fun)
  (setf fun (if procedure?
                #'compile-procedure-call
                #'compile-function-call))  
  (unless env
    (setf env (gensym "RESULT")
          flag t))
  (setf body `(progn
                ,@(loop for binding in (car args)
                        collecting (compile-in-env-binding binding schema env))
                ,(compile-term (second args) schema env fun  )))
  (if flag
      `(let (,env),body)
      body))


(defun compile-in-env-binding (binding schema env)
  `(setf ,env (bind ,(car binding),(compile-term (cdr binding)schema env), env)))


(defun compile-goal-expression (goal schema env)
  (cond ((member (car goal) '(and or))
         `(cons ',(car goal)
                (list
                 ,@(mapcar #'(lambda (x)(compile-goal-expression x schema env ))
                       (cdr goal)))))
        ((eq (car goal) 'not)
         `(list 'not ,(compile-goal-expression (second goal) schema env)))
       (t
        ;;goal is a basic relational expression
        `(cons ',(car goal)
               (list
	        ,@(mapcar #'(lambda (x)
                              (compile-expression-argument x schema env nil t))
                          (cdr goal)))))))


;;;
(defun compile-expression-argument (arg schema env &optional inside-ocml-eval? top-level?)
  (cond ((atom arg)
         (if (variable? arg)
	     (let ((value (cdr (assoc arg schema))))
               (if (and inside-ocml-eval? value) ;; to be compatible with backcmp
                   `(list 'quote ,value)
		 (or value arg)))
           (list 'quote arg)))
         ((eq (car arg) *ocml-eval-macro-symbol*)
          `(list *ocml-eval-macro-symbol*
                 ,(compile-expression-argument 
                    (second arg) schema env t)))
         ((eq (car arg)'lambda)
          `(list 'lambda 
             ,(compile-expression-argument (second arg)schema env)
             ,(compile-expression-argument (third arg)schema env inside-ocml-eval?)))
         (top-level?
          (if (ocml-function? (car arg))
              `(cons ,(compile-expression-argument (car arg) schema env nil)
                     ,(compile-expression-argument (cdr arg) schema env t))
              `(cons ,(compile-expression-argument (car arg) schema env inside-ocml-eval?)
                     ,(compile-expression-argument (cdr arg) schema env inside-ocml-eval?))))
	 (t
	 `(cons ,(compile-expression-argument (car arg) schema env inside-ocml-eval?)
                ,(compile-expression-argument (cdr arg) schema env inside-ocml-eval?)))))

;;;COMPILE-TERM ---Compiles a OCML functional expression.  The idea is that it 
;;;transforms it into something which can be given as argument to EVAL.
(defun compile-term (term schema env &optional (fun #'compile-function-call) )
  (cond  ((consp term)
          (if (eq (car term) 'lambda)
              `(list 'lambda
		     ,(compile-expression-argument (second term) schema env)
		     ,(compile-expression-argument (third term) schema env fun))
            (funcall fun (car term)(cdr term) schema env  )))
         ((variable? term)
          (let* ((value (cdr (assoc term schema))) ;; to be compatible with backcmp
                 (result 
			 (or value 
			     `(ocml-eval-var-or-error ,term ,env))))
            result))
         (t
          (list 'quote term))))


(defun compile-control-term (term schema env)
  (compile-term term schema env #'compile-procedure-call))
     
  
  

(defun compile-all-funs&procs (&optional verbose force-p)
  (maphash #'(lambda (key fun)
               (declare (ignore key))
               (compile-f-or-p fun verbose force-p))
           *defined-functions*))
    

;;;***********************************************************************
 
;;;COMPILE-ALL-DEFINITIONS --Call this function to compile all functions
;;;and inference actions (tasks are not compiled yet!).  If <force-p> is T
;;;then functions and tasks are recompiled even if there is no apparent need to 
;;;do it.

;;If  nocompile = T, then it does not compile lisp

(defun  compile-all-definitions (&optional verbose force-p in-line (nocompile *nocompile*))
  (setf *compiled* t)
  (setf (symbol-function 'prove)  (symbol-function 'prove-template-cmp))
  (compile-bc-rules nocompile in-line)
  (compile-all-funs&procs verbose force-p))


;;; REMOVE-ALL-DEFS-FROM-LISP ---Removes compiled bc-rules and recompiles tasks and funs
(defun remove-all-defs-from-lisp ()
  (setf *compiled* nil)
  (setf (symbol-function 'prove)  (symbol-function 'prove-template-int))
  (clear-all-compiled-bc-rule-code)
  ;;;;;;(compile-all-tasks nil t)
  (compile-all-funs&procs nil t))

(defun use-compiled-defs ()
     (setf *compiled* t)
     (setf (symbol-function 'prove)  (symbol-function 'prove-template-cmp)))
     
(defun use-interpreted-defs ()
     (setf *compiled* nil)
     (setf (symbol-function 'prove)  (symbol-function 'prove-template-int)))

;;;CFUN --This function is useful to see what the result of the compilation
;;;process looks like. <name> is the name of a function or an inference.  In the latter
;;;case <inference?> should be T.  This function does not change the structure of the
;;;function (i.e. it does not 'really' compiles it).
(defun cfun (name)
  (let ((f (get-function name)))
        (with-slots (body schema free-vars-in-body) f
          (if (procedure-p f)
              (pprint (compile-procedure-body body schema free-vars-in-body))
              (pprint (compile-function-body body schema free-vars-in-body))))))

;;;SET-LISP-FUN ---This function can be used to (re)compile a function
(defun set-lisp-fun (name &optional (compile-to-binary? t))
  (let ((f (get-function name)))
    (with-slots (body schema lisp-fun free-vars-in-body)f
      (setf lisp-fun
        (funcall (if compile-to-binary?
                     #'(lambda (x)(compile-attachment x name))
                     #'eval)
                 (if (procedure-p f)
                     (compile-procedure-body body schema free-vars-in-body)
                     (compile-function-body body schema free-vars-in-body)))))))
