package org.infrawebs.axiom.editor;

import org.eclipse.draw2d.Cursors;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Tool;
import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.tools.ConnectionCreationTool;
import org.eclipse.jface.resource.ImageDescriptor;
import org.infrawebs.axiom.model.ManhattanConnectionModel;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;

/**
 * An auto-deactivating connection creation currentTool. After a connection is
 * completed, this currentTool deactivates itself and the default currentTool is selected.
 * Manhattan connections are used by default. 
 */
public class AxiomConnectionTool extends ConnectionCreationTool
{
	private static final SimpleFactory CONNECTION_FACTORY =
			new SimpleFactory(ManhattanConnectionModel.class);

	public AxiomConnectionTool() {
		super(CONNECTION_FACTORY);
		setUnloadWhenFinished(true);
		setDisabledCursor(Cursors.NO);
	}

	private EditPart autoSource = null;
	private boolean overrideClickedPart = false;
	
	public AxiomConnectionTool(EditPart autoSource) {
		this();
		AxiomEditor editor = AxiomEditorPlugin.getAxiomEditor();
		GraphicalViewer viewer = (GraphicalViewer)editor.getAdapter(GraphicalViewer.class);
		setViewer(viewer);
		EditDomain editDomain = (EditDomain)editor.getAdapter(EditDomain.class);
		editDomain.setActiveTool(this);

		// Simulate the pressing of the left mouse button.
		// handleButtonDown(1) also calls updateTargetUnderMouse() which
		// would normally set the target EditPart to null is most cases.
		// To avoid that, updateTargetUnderMouse() is patched below.
		this.autoSource = autoSource;
		this.overrideClickedPart = true;
		handleButtonDown(1);
	}

	@Override
	protected boolean updateTargetUnderMouse() {
		boolean changed = super.updateTargetUnderMouse();
		// If the source edit part was set programmatically, force the
		// target EditPart to be that fixed source (instead of null).
		// Thus, handleButtonDown(1) will successfully start a connection.
		if (this.overrideClickedPart) {
			setTargetEditPart(this.autoSource);
			// The source object will be selected. Then the selection of the
			// destination object will begin. The process must not be
			// altered in any way so the flag is cleared.
			this.overrideClickedPart = false;
		}
		return changed;
	}

	/**
	 * A palette entry returning an auto-deactivating connection creation currentTool
	 * which uses Manhattan connections. 
	 */
	public static class Entry extends ConnectionCreationToolEntry {
		public Entry(String label, String shortDesc,
				ImageDescriptor iconSmall, ImageDescriptor iconLarge) {
			super(label, shortDesc, null, iconSmall, iconLarge);
		}

		@Override
		public Tool createTool() {
			return new AxiomConnectionTool();
		}
	}

}


