package org.infrawebs.axiom.editor;

import java.io.File;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.AlignmentAction;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.PrintAction;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.SaveAction;
import org.eclipse.gef.ui.actions.SelectAllAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.gef.ui.parts.SelectionSynchronizer;
import org.eclipse.gef.ui.properties.UndoablePropertySheetEntry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.TransferDropTargetListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.infrawebs.axiom.editor.actions.ConnectToExistingAction;
import org.infrawebs.axiom.editor.actions.RenameAction;
import org.infrawebs.axiom.editor.actions.connections.InsertOperatorAndAction;
import org.infrawebs.axiom.editor.actions.connections.InsertOperatorNotAction;
import org.infrawebs.axiom.editor.actions.connections.InsertOperatorOrAction;
import org.infrawebs.axiom.editor.actions.contents.CreateInstanceAction;
import org.infrawebs.axiom.editor.actions.contents.CreateOperatorAndAction;
import org.infrawebs.axiom.editor.actions.contents.CreateOperatorNotAction;
import org.infrawebs.axiom.editor.actions.contents.CreateOperatorOrAction;
import org.infrawebs.axiom.editor.actions.contents.CreateRelationAction;
import org.infrawebs.axiom.editor.actions.contents.CreateVariableAction;
import org.infrawebs.axiom.editor.actions.memberofs.InvolveInNewRelationAction;
import org.infrawebs.axiom.editor.actions.ontology.LocateInOntoViewAction;
import org.infrawebs.axiom.editor.actions.ontology.LocateTypeInOntoViewAction;
import org.infrawebs.axiom.editor.actions.operators.AddOperandCustomTypeComplexInstanceAction;
import org.infrawebs.axiom.editor.actions.operators.AddOperandCustomTypeValueInstanceAction;
import org.infrawebs.axiom.editor.actions.operators.AddOperandCustomTypeVariableAction;
import org.infrawebs.axiom.editor.actions.operators.AddOperandDefaultTypeValueInstanceAction;
import org.infrawebs.axiom.editor.actions.operators.AddOperandDefaultTypeVariableAction;
import org.infrawebs.axiom.editor.actions.operators.AddOperandRelationAction;
import org.infrawebs.axiom.editor.actions.refinables.AdditionalAttributeValueAction;
import org.infrawebs.axiom.editor.actions.refinables.RefineAndInvolveInRelationAction;
import org.infrawebs.axiom.editor.actions.refinables.RefineByCustomTypeComplexInstanceAction;
import org.infrawebs.axiom.editor.actions.refinables.RefineByCustomTypeValueInstanceAction;
import org.infrawebs.axiom.editor.actions.refinables.RefineByCustomTypeVariableAction;
import org.infrawebs.axiom.editor.actions.refinables.RefineByDefaultTypeValueInstanceAction;
import org.infrawebs.axiom.editor.actions.refinables.RefineByDefaultTypeVariableAction;
import org.infrawebs.axiom.editor.actions.visual.BringToFrontAction;
import org.infrawebs.axiom.editor.actions.visual.HideUnusedAttributes;
import org.infrawebs.axiom.editor.actions.visual.SendToBackAction;
import org.infrawebs.axiom.editor.editparts.AxiomContentsEditPart;
import org.infrawebs.axiom.editor.editparts.AxiomEditPartFactory;
import org.infrawebs.axiom.editor.inputs.NewEditorInput;
import org.infrawebs.axiom.editor.inputs.PathEditorInput;
import org.infrawebs.axiom.outline.AxiomTreeOutlinePage;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.common.Messages;
import org.wsmostudio.ui.editors.WSMOEditor;
import org.wsmostudio.ui.editors.WSMOEditorInput;
import org.wsmostudio.ui.editors.model.AxiomModel;

public class AxiomEditor extends WSMOEditor implements CommandStackListener, ISelectionListener
{
	public static final String EDITOR_ID = "org.infrawebs.axiom.editor.axiomEditor";

	private EditDomain editDomain = new AxiomEditDomain(this);
	private GraphicalViewer graphicalViewer = new ScrollingGraphicalViewer();
	private ActionRegistry actionRegistry = new ActionRegistry();
	private List<String> selectionActions = new ArrayList<String>();
	private List<String> stackActions = new ArrayList<String>();
	private List<String> propertyActions = new ArrayList<String>();
	private SelectionSynchronizer synchronizer = new SelectionSynchronizer();

	private AxiomEditorContents contents = new AxiomEditorContents();

	public AxiomEditor() {
		// Nothing more to initialize.
	}

	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		setSite(site);
		setInput(input);
		this.editDomain.getCommandStack().addCommandStackListener(this);
		getSite().getWorkbenchWindow().getSelectionService().addSelectionListener(this);
		createActions();
		
		initEditorContents(input);

		if (this.contents.getAxiomContentsModel() != null) {
			// Change the title of the editor tab. 
			firePropertyChange(PROP_TITLE);
		}
		else {
			// Init canceled of failed. Create a temporary empty model and close the editor.
			this.contents.initEmpty(input.getName());
			super.getSite().getPage().closeEditor(this, false);
		}
	}

	private void initEditorContents(IEditorInput input) throws PartInitException {
		if (input instanceof NewEditorInput) {
			this.contents.initEmpty(input.getName());
		}
		else if (input instanceof IPathEditorInput) {
			this.contents.initFromFile(((IPathEditorInput)input).getPath());
		}
		else if (input instanceof WSMOEditorInput) {
			AxiomModel studioModel = (AxiomModel)input.getAdapter(AxiomModel.class);
			if (studioModel != null) {
				this.contents.initFromStudio(studioModel);
			}
			else {
				throw new PartInitException("This WSMOEditorInput does not support AxiomModels.");
			}
		}
		else {
			throw new PartInitException("Unexpected editor input " + input.getClass().getName());
		}
	}

	@Override
	public void dispose() {
		getSite().getWorkbenchWindow().getSelectionService().removeSelectionListener(this);
		this.editDomain.getCommandStack().removeCommandStackListener(this);
		this.editDomain.setActiveTool(null);
		this.actionRegistry.dispose();
		super.dispose();
	}

	@Override
	public void createPartControl(Composite parent) {
		this.graphicalViewer.createControl(parent);
		configureGraphicalViewer();
		this.editDomain.addViewer(this.graphicalViewer);
		this.synchronizer.addViewer(this.graphicalViewer);
		getSite().setSelectionProvider(this.graphicalViewer);

		this.graphicalViewer.setContents(this.contents.getAxiomContentsModel());
	}

	protected void configureGraphicalViewer() {
		this.graphicalViewer.getControl().setBackground(ColorConstants.listBackground);
		ScalableRootEditPart rootEditPart = new ScalableRootEditPart();
		this.graphicalViewer.setRootEditPart(rootEditPart);
		configureZoomManager(rootEditPart);
		this.graphicalViewer.setEditPartFactory(new AxiomEditPartFactory());

		// Context menu support.
		ContextMenuProvider cmProvider = new ContextMenuProviderImpl(this.actionRegistry, this.graphicalViewer);
		this.graphicalViewer.setContextMenu(cmProvider);
		//getSite().registerContextMenu(cmProvider, this.graphicalViewer); // ??????????????????		

		// Drag&drop support.
		this.graphicalViewer.addDropTargetListener((TransferDropTargetListener)
				new OntologyTransferDropTargetListener(this.graphicalViewer));		
	}

	private void configureZoomManager(ScalableRootEditPart rootEditPart) {
		ZoomManager manager = rootEditPart.getZoomManager();
		double[] zoomLevels = new double[] { 0.25, 0.5, 0.75, 1.0, 1.5, 2.0,
				2.5, 3.0, 4.0, 5.0, 10.0, 20.0 };
		manager.setZoomLevels(zoomLevels);

		ArrayList<String> zoomContributions = new ArrayList<String>();
		zoomContributions.add(ZoomManager.FIT_ALL);
		zoomContributions.add(ZoomManager.FIT_HEIGHT);
		zoomContributions.add(ZoomManager.FIT_WIDTH);
		manager.setZoomLevelContributions(zoomContributions);

		this.actionRegistry.registerAction(new ZoomInAction(manager));
		this.actionRegistry.registerAction(new ZoomOutAction(manager));
	}

	public AxiomEditorContents getContents() {
		return this.contents;
	}

	@Override
	public String getTitle() {
		IEditorInput input = getEditorInput();
		if (input == null) {
			return super.getTitle();
		}
		return input.getName();
	}

	public AxiomContentsEditPart getAxiomContentsEditPart() {
		return (AxiomContentsEditPart)this.graphicalViewer.getContents();
	}

	private void addAction(IAction action) {
		// Add a selection action.
		this.actionRegistry.registerAction(action);
		this.selectionActions.add(action.getId());
	}

	protected void createActions() {
		// Always available actions.
		this.actionRegistry.registerAction(new SelectAllAction(this));
		this.actionRegistry.registerAction(new PrintAction(this));

		// Command stack actions.
		IAction undoAction = new UndoAction(this);
		this.actionRegistry.registerAction(undoAction);
		this.stackActions.add(undoAction.getId());

		IAction redoAction = new RedoAction(this);
		this.actionRegistry.registerAction(redoAction);
		this.stackActions.add(redoAction.getId());

		IAction saveAction = new SaveAction(this);
		saveAction.setEnabled(false);
		this.actionRegistry.registerAction(saveAction);
		this.stackActions.add(saveAction.getId());

		updateActions(this.stackActions);

		// Property actions.
		this.propertyActions.add(saveAction.getId());
		
		updateActions(this.propertyActions);

		// Selection actions.
		addAction(new DeleteAction((IWorkbenchPart)this));

		addAction(new HideUnusedAttributes(this));		
		addAction(new BringToFrontAction(this));
		addAction(new SendToBackAction(this));
		addAction(new RenameAction(this));
		addAction(new CreateVariableAction(this));
		addAction(new CreateInstanceAction(this));
		addAction(new CreateRelationAction(this));
		addAction(new CreateOperatorOrAction(this));
		addAction(new CreateOperatorAndAction(this));
		addAction(new CreateOperatorNotAction(this));
		addAction(new InsertOperatorAndAction(this));		
		addAction(new InsertOperatorOrAction(this));
		addAction(new InsertOperatorNotAction(this));		
		
		addAction(new RefineByDefaultTypeVariableAction(this));		
		addAction(new RefineByDefaultTypeValueInstanceAction(this));
		addAction(new RefineByCustomTypeVariableAction(this));
		addAction(new RefineByCustomTypeValueInstanceAction(this));
		addAction(new RefineByCustomTypeComplexInstanceAction(this));
		addAction(new RefineAndInvolveInRelationAction(this));
		addAction(new AdditionalAttributeValueAction(this));		
		
		addAction(new AddOperandDefaultTypeVariableAction(this));
		addAction(new AddOperandDefaultTypeValueInstanceAction(this));
		addAction(new AddOperandCustomTypeVariableAction(this));
		addAction(new AddOperandCustomTypeValueInstanceAction(this));
		addAction(new AddOperandCustomTypeComplexInstanceAction(this));
		addAction(new AddOperandRelationAction(this));
		
		addAction(new InvolveInNewRelationAction(this));
		addAction(new ConnectToExistingAction(this));
		addAction(new LocateInOntoViewAction(this));		
		addAction(new LocateTypeInOntoViewAction(this));		
		addAction(new RenameAction(this));

		addAction(new AlignmentAction((IWorkbenchPart)this, PositionConstants.LEFT));
		addAction(new AlignmentAction((IWorkbenchPart)this, PositionConstants.CENTER));
		addAction(new AlignmentAction((IWorkbenchPart)this, PositionConstants.RIGHT));
		addAction(new AlignmentAction((IWorkbenchPart)this, PositionConstants.TOP));
		addAction(new AlignmentAction((IWorkbenchPart)this, PositionConstants.MIDDLE));
		addAction(new AlignmentAction((IWorkbenchPart)this, PositionConstants.BOTTOM));

		updateActions(this.selectionActions);
	}

	/** 
	 * Returns the adapter for the specified key.
	 * 
	 * <P><EM>IMPORTANT</EM> certain requests, such as the property sheet, may be made before
	 * or after {@link #createPartControl(Composite)} is called. The order is unspecified by
	 * the Workbench.
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	@Override
	public Object getAdapter(Class type) {
		if (type == IPropertySheetPage.class) {
			PropertySheetPage page = new PropertySheetPage();
			page.setRootEntry(new UndoablePropertySheetEntry(this.editDomain.getCommandStack()));
			return page;
		}
		if (type == IContentOutlinePage.class) {
			return new AxiomTreeOutlinePage(this.actionRegistry);
		}
		if (type == GraphicalViewer.class) {
			return this.graphicalViewer;
		}
		if (type == EditPart.class) {
			return this.graphicalViewer.getRootEditPart();
		}
		if (type == IFigure.class) {
			return ((GraphicalEditPart)this.graphicalViewer.getRootEditPart()).getFigure();
		}
		if (type == ZoomManager.class) {
			return ((ScalableRootEditPart)this.graphicalViewer.getRootEditPart()).getZoomManager();
		}
		if (type == EditDomain.class) {
			return this.editDomain;
		}
		if (type == CommandStack.class) {
			return this.editDomain.getCommandStack();
		}
		if (type == SelectionSynchronizer.class) {
			return this.synchronizer;
		}
		if (type == ActionRegistry.class) {
			return this.actionRegistry;
		}
		return super.getAdapter(type);
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
		if (this.contents.getFilePath() != null) {
			// Use the file system only. Save the file and do not touch WSMO Studio.
			if (this.contents.saveToFile(this.contents.getFilePath())) {
				this.editDomain.getCommandStack().markSaveLocation();
				notifyDirty();
			}
		}
		else if (this.contents.getStudioModel() != null) {
			// Save to WSMO Studio. The super method calls doTryToSave().
			super.doSave(monitor);
		}
		else {
			// Neither a file nor a studio model is present. Save to a new file.
			doSaveAs();
		}
	}

	@Override
	protected void doTryToSave() throws Exception {
        this.contents.saveToStudio();
	}

	/**
	 * Saves the model to an *.axiom file and detach from WSMO Studio. 
	 */
	@Override
	public void doSaveAs() {
		// Show a save as dialog and let the user select a new file name.
		String fileName = getSaveAsDialog().open();
		if (fileName == null) {
			return;
		}

		if (new File(fileName).exists() && false == Messages.showQuestion(
				fileName + " already exists.\nDo you want to replace it?")) {
			return;
		}

		IPath oldPath = this.contents.getFilePath();
		IPath newPath = new Path(fileName);
		if (this.contents.saveToFile(newPath)) {
			this.editDomain.getCommandStack().markSaveLocation();
			notifyDirty();

			// Create a new editor input.
			if (!newPath.equals(oldPath)) {
				setInput(new PathEditorInput(newPath));
				firePropertyChange(PROP_INPUT);
				firePropertyChange(PROP_TITLE);
			}
		}
	}

	@Override
	public boolean isSaveAsAllowed() {
		return true;
	}

	@Override
	protected void updateFromModel() {
		Messages.showInformation("TODO: AxiomEditor.updateFromModel()");
	}

	@Override
	public boolean isDirty() {
		//super.isDirty() It doesn't matter wether the studio object is dirty.
		return this.editDomain.getCommandStack().isDirty();
	}

	@Override
	public void setFocus() {
        super.setFocus();
		this.graphicalViewer.getControl().setFocus();
	}

	boolean stackChanging = false;

	public void commandStackChanged(EventObject event) {
		// This guard is necessary because markSaveLocation() recursively calls commandStackChanged().
		if (this.stackChanging) {
			return;
		}
		this.stackChanging = true;

		try {
			// Rebuild the LogicalExpression.
			// Update the WSMO Studio object if there is one.
			this.contents.setChanged();

			// TODO: Implement "revert to original on exit without save".
			// Workaround:
			// If we are editing a WSMO Studio object, changes have already been made
			// to the TopEntity. The only way to rollback is to UNDO to the beginning.
			// To avoid confusion, pretend that the axiom is always saved.
			// When the user closes this editor he won't be asked for confirmation
			// because we have no rollback procedure in case he says "No".
			if (this.contents.getStudioModel() != null) {
				this.editDomain.getCommandStack().markSaveLocation();
			}
			notifyDirty();
	
			updateActions(this.stackActions);
		}
		finally {
			this.stackChanging = false;
		}
	}

	@Override
	protected void firePropertyChange(int property) {
		super.firePropertyChange(property);
		updateActions(this.propertyActions);
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		// If not the active editor, ignore selection changed.
		if (this.equals(getSite().getPage().getActiveEditor())) {
			updateActions(this.selectionActions);
		}
	}

	/**
	 * A convenience method for updating a set of actions defined by the given List of action
	 * IDs. The actions are found by looking up the ID in the action registry. If the corresponding
	 * action is an {@link UpdateAction}, it will have its <code>update()</code> method called.
	 * @param actionIds the list of IDs to update
	 */
	private void updateActions(List<String> actionIds) {
		for (String id: actionIds) {
			IAction action = this.actionRegistry.getAction(id);
			if (action instanceof UpdateAction) {
				((UpdateAction)action).update();
			}
		}
	}

	private static FileDialog saveAsDialog = null;

	private static FileDialog getSaveAsDialog() {
		if (saveAsDialog == null) {
			saveAsDialog = new FileDialog(AxiomEditorPlugin.getDefault().getWorkbench().
					getActiveWorkbenchWindow().getShell(), SWT.SAVE);
			saveAsDialog.setText("Save file as...");
			saveAsDialog.setFilterNames(new String[] {
					"Axioms (*.axiom)",
					"All files (*.*)"
			});
			saveAsDialog.setFilterExtensions(new String[] {
					"*.axiom", //$NON-NLS-1$
					"*.*" //$NON-NLS-1$
			});
		}
		return saveAsDialog;
	}

}
