package org.infrawebs.axiom.editor.actions;

import java.util.List;

import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.AxiomEditor;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.AxiomContentsModel;

public abstract class AxiomAction extends SelectionAction
{
	private AxiomEditor editor = null; 

	public AxiomAction(IWorkbenchPart part) {
		super(part);
		this.editor = (part instanceof AxiomEditor) ? (AxiomEditor)part : null;
	}

	@Override
	public void init() {
		setId(this.getClass().getName());
		setText(this.getCaption());
		Image image = this.getImage();
		if (image != null) {
			setImageDescriptor(ImageDescriptor.createFromImage(image));
		}
	}
	
	protected abstract String getCaption();

	protected abstract Image getImage();
	
	public abstract boolean calculateVisible();
	
	@Override
	protected boolean calculateEnabled() {
		return calculateVisible();
	}

	protected AbstractEditPart getSelectedEditPart() {
		List list = getSelectedObjects();
		return (list.size() == 1) && (list.get(0) instanceof AbstractEditPart)
				? (AbstractEditPart)list.get(0) : null;
	}

	protected AbstractModel getSelectedModel() {
		AbstractEditPart editPart = getSelectedEditPart();
		return (editPart != null) ? (AbstractModel)editPart.getModel() : null;
	}

	public AxiomContentsModel getContentsModel() {
		return this.editor.getContents().getAxiomContentsModel();
	}
}
