package org.infrawebs.axiom.editor.actions.connections;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.editor.editparts.AxiomConnectionEditPart;
import org.infrawebs.axiom.model.ConnectionModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.model.OperatorModel;
import org.infrawebs.axiom.model.commands.ConnectionReconnectCommand;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.axiom.settings.ImageSettings;


public abstract class InsertOperatorAction extends AxiomAction
{
	public InsertOperatorAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		return (getSelectedModel() instanceof ConnectionModel);
	}
	
	protected boolean isAnyEndSameOperator(ConnectionModel conn) {
		if (conn.getSource() instanceof OperatorModel) {
			OperatorModel operator = (OperatorModel)conn.getSource();
			if (operator.getOperatorType() == getOperatorType()) {
				return true;
			}
		}
		if (conn.getTarget() instanceof OperatorModel) {
			OperatorModel operator = (OperatorModel)conn.getTarget(); 
			if (operator.getOperatorType() == getOperatorType()) {
				return true;
			}
		}
		return false;
	}

	protected abstract int getOperatorType();
	
	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}
		AxiomConnectionEditPart connectionEditPart = (AxiomConnectionEditPart)this.getSelectedEditPart();
		ConnectionModel connectionModel = (ConnectionModel)connectionEditPart.getModel();

		OperatorModel operatorModel = getContentsModel().getFactory().createOperator(getOperatorType(), null);
		Command createCommand = ModelOperations.getCreateCommand(operatorModel);
		operatorModel.setConstraint(ModelOperations.getMiddleOf(connectionEditPart));		
		
		ModelWithConnections oldTarget = connectionModel.getTarget();
		ConnectionReconnectCommand reconnectCommand = new ConnectionReconnectCommand(connectionModel);
		reconnectCommand.setNewTarget(operatorModel);
		
		Command connectCommand = ModelOperations.getConnectionCommand(operatorModel, oldTarget);

		CompoundCommand compoundCommand = new CompoundCommand();		
		compoundCommand.add(createCommand);
		compoundCommand.add(reconnectCommand);
		compoundCommand.add(connectCommand);
		ModelOperations.executeCommand(compoundCommand);
	}

	@Override
	protected String getCaption() {
		return "Insert operator " + OperatorModel.getOperatorTypeAsString(getOperatorType()); 
	}

	@Override
	protected Image getImage() {
		return ImageSettings.getImage(ImageSettings.OPERATOR);
	}

}
