package org.infrawebs.axiom.editor.actions.ontology;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.InstanceModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.model.RelationModel;
import org.infrawebs.axiom.model.VariableModel;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.axiom.settings.ImageSettings;
import org.omwg.ontology.OntologyElement;

public class LocateInOntoViewAction extends AxiomAction
{
	public LocateInOntoViewAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = this.getSelectedModel();
		return (model instanceof MemberOfModel)
				|| (model instanceof RefinableModel)
				|| (model instanceof RelationModel);
	}
	
	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}
		AbstractModel model = this.getSelectedModel();
		OntologyElement element = null;
		if (model instanceof VariableModel) {
			element = ((VariableModel)model).getConcept();
		} 
		else if (model instanceof InstanceModel) {
			element = ((InstanceModel)model).getInstance();
		}
		else if (model instanceof RelationModel) {
			element = ((RelationModel)model).getRelation();
		}
		else if (model instanceof RefinableModel) {
			element = ((RefinableModel)model).getRefinable();
		}
		
		if (element != null) {
			AxiomEditorPlugin.ontoLoader.jumpToDefinition(element);
		}
	}

	@Override
	protected String getCaption() {
		return "Locate in Ontology";
	}

	@Override
	protected Image getImage() {
		return ImageSettings.getImage(ImageSettings.LOCATE);
	}

}
