package org.infrawebs.axiom.editor.actions.operators;

import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.ontology.SelectFromOntologyAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.OperatorModel;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.axiom.operations.SemanticConsistence;
import org.infrawebs.ontology.OntoFilter;
import org.omwg.ontology.OntologyElement;

public abstract class AddOperandCustomTypeAction extends SelectFromOntologyAction
{
	protected RefinableModel chainOrigin = null;

	public AddOperandCustomTypeAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		if (!(model instanceof OperatorModel)) {
			return false;
		}
		OperatorModel operator = (OperatorModel)getSelectedModel();

		// chainOrigin is used by the sub-classes and by createOntoFilter(). It must be initialized here.
		List<RefinableModel> origins = operator.listChainOrigins();
		this.chainOrigin = (origins.size() > 0) ? origins.get(0) : null;
		
		return SemanticConsistence.allowMoreOperands(operator);
	}

	@Override
	protected OntoFilter createOntoFilter() {
		OntoFilter ontoFilter = new OntoFilter(false);

		// Show only elements which are compatible with the types
		// of the origins of the connection chains.
		if (this.chainOrigin != null) {
			ontoFilter.baseTypes = this.chainOrigin.getRefinable().listTypes();
		}
		return ontoFilter;
	}

	@Override
	protected void onOkPressed(OntologyElement ontologyElement) {
		MemberOfModel memberOfModel = (MemberOfModel)getContentsModel().getFactory().createAxiomElement(ontologyElement);
		if (memberOfModel == null) {
			return;
		}

		// Place the new figure to the right of the operator.
		OperatorModel sourceModel = (OperatorModel)getSelectedModel();
		Rectangle rect = (Rectangle)sourceModel.getConstraint();
		memberOfModel.setConstraint(new Rectangle(rect.x + 100, rect.y, -1, -1));
		// Point to the same VarNameModel as the refinable
		// which is the origin of the connection chain.
//		if (this.chainOrigins.size() > 0) {
//			RefinableModel refinable = (RefinableModel)this.chainOrigins.get(0);
//			memberOfModel.setVarNameModel(refinable.getVarNameModel());
//		}

		ModelOperations.createAndConnect(
				memberOfModel, sourceModel, memberOfModel);
	}

}
