package org.infrawebs.axiom.editor.actions.operators;

import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.editor.editparts.OperatorEditPart;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.OperatorModel;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.axiom.operations.SemanticConsistence;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.omwg.ontology.Concept;
import org.omwg.ontology.Type;

public abstract class AddOperandDefaultTypeAction extends AxiomAction
{
	protected RefinableModel chainOrigin = null;

	public AddOperandDefaultTypeAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		if (!(model instanceof OperatorModel)) {
			return false;
		}
		OperatorModel operator = (OperatorModel)model;
		List<RefinableModel> origins = operator.listChainOrigins();
		this.chainOrigin = (origins.size() > 0) ? origins.get(0) : null;
		// The default type of the operand can be determined only if
		// the connection chain originates from a RefinableModel.
		return SemanticConsistence.allowMoreOperands(operator) &&
				(this.chainOrigin != null);
	}
	
	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}

		OperatorEditPart sourcePart = (OperatorEditPart)this.getSelectedEditPart();
		OperatorModel sourceModel = (OperatorModel)sourcePart.getModel();
		Type defaultType = this.chainOrigin.getRefinable().getFirstType();
		Concept concept = AxiomEditorPlugin.ontoStore.liftTypeToConcept(defaultType); 
		MemberOfModel memberOfModel = (MemberOfModel)getContentsModel().getFactory().createAxiomElement(concept);
		if (memberOfModel == null) {
			return;
		}

		// Place the new figure to the right of the operator.
		Rectangle rect = (Rectangle)sourceModel.getConstraint();
		memberOfModel.setConstraint(new Rectangle(rect.x + 100, rect.y, -1, -1));
		// Point to the same VarNameModel as the refinable
		// which is the origin of the connection chain.
//		memberOfModel.setVarNameModel(refinable.getVarNameModel());
		
		ModelOperations.createAndConnect(
				memberOfModel, sourceModel, memberOfModel);
	}

}
