package org.infrawebs.axiom.editor.actions.operators;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.ontology.SelectFromOntologyAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.OperatorModel;
import org.infrawebs.axiom.model.RelationModel;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.axiom.operations.SemanticConsistence;
import org.infrawebs.ontology.OntoFilter;
import org.infrawebs.ontology.ui.OntoImages;
import org.omwg.ontology.OntologyElement;

public class AddOperandRelationAction extends SelectFromOntologyAction
{
	public AddOperandRelationAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		if (model instanceof OperatorModel) {
			OperatorModel operator = (OperatorModel)model;
			if (SemanticConsistence.allowMoreOperands(operator)) {
				// A relation operand can be attached only if the connection
				// chain does NOT originate from a RefinableModel.
				return (operator.listChainOrigins().size() == 0);
			}
		}
		return false;
	}

	@Override
	protected String getCaption() {
		return "Add a relation operand";
	}

	@Override
	protected Image getImage() {
		return OntoImages.getImage(OntoImages.RELATION);
	}

	@Override
	protected String getSelectionDialogTitle() {
		return "Please, select a relation.";
	}

	@Override
	protected OntoFilter createOntoFilter() {
		OntoFilter ontoFilter = new OntoFilter(false);
		ontoFilter.showRelations = true;
		ontoFilter.selectRelations = true;
		return ontoFilter;
	}

	@Override
	protected void onOkPressed(OntologyElement ontologyElement) {
		RelationModel relationModel = (RelationModel)getContentsModel().getFactory().createAxiomElement(ontologyElement);
		if (relationModel == null) {
			return;
		}
		
		// Place the new figure to the right of the operator.
		OperatorModel sourceModel = (OperatorModel)getSelectedModel();
		Rectangle rect = (Rectangle)sourceModel.getConstraint();
		relationModel.setConstraint(new Rectangle(rect.x + 100, rect.y, -1, -1));

		ModelOperations.createAndConnect(
				relationModel, sourceModel, relationModel);
	}
}
