package org.infrawebs.axiom.editor.actions.refinables;

import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.memberofs.InvolveInNewRelationAction;
import org.infrawebs.axiom.editor.actions.ontology.SelectFromOntologyAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.ParameterModel;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.model.RelationModel;
import org.infrawebs.axiom.model.VarNameModel;
import org.infrawebs.axiom.model.commands.VarNameChangeCommand;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.ontology.OntoFilter;
import org.infrawebs.ontology.shells.ParameterShell;
import org.infrawebs.ontology.ui.OntoImages;
import org.omwg.ontology.OntologyElement;

public class RefineAndInvolveInRelationAction extends SelectFromOntologyAction
{
	public RefineAndInvolveInRelationAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		return (model instanceof RefinableModel) &&
				(((RefinableModel)model).getRefiningModel() == null);
	}

	@Override
	protected String getCaption() {
		return "Involve in relation";
	}

	@Override
	protected Image getImage() {
		return OntoImages.getImage(OntoImages.RELATION);
	}

	@Override
	protected String getSelectionDialogTitle() {
		return "Please, select a compatible relation parameter.";
	}

	@Override
	protected OntoFilter createOntoFilter() {
		OntoFilter ontoFilter = new OntoFilter(false);
		RefinableModel refinable = (RefinableModel)getSelectedModel();
		ontoFilter.baseTypes = refinable.getRefinable().listTypes();
		ontoFilter.showRelations = true;
		ontoFilter.selectParameters = true;
		return ontoFilter;
	}

	/**
	 * Since this is a batch mode for several other actions and commands
	 * (RefineByDefaultTypeAction, InvolveInNewRelationAction, ChangeCompactCommand),
	 * directly reuse some or their implementations.
	 */ 
	@Override
	protected void onOkPressed(OntologyElement ontologyElement) {
		// Create a model for the relation which the selected parameter belongs to.
		ParameterShell paramInfo = (ParameterShell)ontologyElement;
		RelationModel relationModel = getContentsModel().getFactory().createRelation(paramInfo);
		if (relationModel == null) {
			return;
		}
		// Create a compact variable to bridge the active refinable and the new parameter.
		RefinableModel refinableModel = (RefinableModel)getSelectedModel();
		MemberOfModel memberOfModel = RefineByDefaultTypeAction.createDefaultRefiningModel(
				getContentsModel(), refinableModel);
		if (memberOfModel == null) {
			return;
		}
		memberOfModel.setCompact(true);
		ParameterModel paramModel = InvolveInNewRelationAction.findParameterModel(relationModel, paramInfo);

		CompoundCommand compoundCommand = new CompoundCommand();
		// 1. Create both new elements.
		compoundCommand.add(ModelOperations.getCreateCommand(relationModel));
		compoundCommand.add(ModelOperations.getCreateCommand(memberOfModel));
		// 2. Attach the same varNameModel to all elements.
		VarNameModel varNameModel = refinableModel.createVarNameModel();
		compoundCommand.add(new VarNameChangeCommand(refinableModel, varNameModel));
		compoundCommand.add(new VarNameChangeCommand(paramModel, varNameModel));
		compoundCommand.add(new VarNameChangeCommand(memberOfModel, varNameModel));
		// 3. Create both connections.
		compoundCommand.add(ModelOperations.getConnectionCommand(refinableModel, memberOfModel));
		compoundCommand.add(ModelOperations.getConnectionCommand(paramModel, memberOfModel));
		ModelOperations.executeCommand(compoundCommand);
	}
}
