package org.infrawebs.axiom.editor.actions.refinables;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.operations.SemanticConsistence;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.axiom.settings.ImageSettings;
import org.infrawebs.ontology.OntoFilter;
import org.omwg.ontology.Concept;
import org.omwg.ontology.OntologyElement;
import org.omwg.ontology.Type;

public class RefineByCustomTypeVariableAction extends RefineByCustomTypeAction
{
	public RefineByCustomTypeVariableAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		// Show action only for refinables of complex types.
		return super.calculateVisible() &&
				!SemanticConsistence.isValueType((RefinableModel)getSelectedModel());
	}
	
	@Override
	protected String getCaption() {
		return "Refine by a custom type variable...";
	}

	@Override
	protected Image getImage() {
		return ImageSettings.getImage(ImageSettings.VARIABLE);
	}

	@Override
	protected String getSelectionDialogTitle() {
		return "Please, select a type for the variable. The type is always a concept.";
	}

	@Override
	protected OntoFilter createOntoFilter() {
		OntoFilter ontoFilter = super.createOntoFilter();
		ontoFilter.showConcepts = true;
		ontoFilter.showAttributes = true;
		ontoFilter.selectConcepts = true;
		return ontoFilter;
	}
	
	@Override
	public void run() {

		// try to locate the definition of the type of the selected refinable 
		RefinableModel refinableModel = (RefinableModel)getSelectedModel();
		Type defaultType = refinableModel.getRefinable().getFirstType();
		Concept concept = AxiomEditorPlugin.ontoStore.liftTypeToConcept(defaultType); 
		OntologyElement definition = AxiomEditorPlugin.ontoStore.findDefinition(concept);
		
		if (definition == null) {
			// Try to load the ontology in which this concept is defined.
			AxiomEditorPlugin.ontoLoader.jumpToDefinition(concept);
		}

		// proceed as usual
		super.run();
	}
}
