package org.infrawebs.axiom.editor.editparts;

import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPart;
import org.infrawebs.axiom.model.ArrowConnectionModel;
import org.infrawebs.axiom.model.AttributeListModel;
import org.infrawebs.axiom.model.AttributeModel;
import org.infrawebs.axiom.model.AxiomContentsModel;
import org.infrawebs.axiom.model.AxiomRootModel;
import org.infrawebs.axiom.model.InstanceModel;
import org.infrawebs.axiom.model.ManhattanConnectionModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.OperatorModel;
import org.infrawebs.axiom.model.ParameterListModel;
import org.infrawebs.axiom.model.ParameterModel;
import org.infrawebs.axiom.model.RelationModel;
import org.infrawebs.axiom.model.ValueInstanceModel;
import org.infrawebs.axiom.model.VarNameModel;
import org.infrawebs.axiom.model.VariableModel;
import org.infrawebs.common.Messages;

public class AxiomEditPartFactory implements EditPartFactory {

	public EditPart createEditPart(EditPart context, Object model) {
		EditPart part = null;

		// WARNING !!!
		// THE ORDER OF THE STATEMENTS IS CRUCIAL
		// OBSERVE THE CLASS HIERARCHY !!!
		
		// diagram elements
		if (model instanceof VarNameModel)
			part = new VarNameEditPart();
		else if (model instanceof InstanceModel)
			part = new InstanceEditPart();
		else if (model instanceof ValueInstanceModel)
			part = new ValueInstanceEditPart();
		else if (model instanceof VariableModel)
			part = new VariableEditPart();
		else if (model instanceof RelationModel)
			part = new RelationEditPart();
		else if (model instanceof AttributeModel)
			part = new AttributeEditPart();
		else if (model instanceof ParameterModel)
			part = new ParameterEditPart();

		// operator
		else if (model instanceof OperatorModel)
			part = new OperatorEditPart();
		else if (model instanceof AxiomRootModel)
			part = new AxiomRootEditPart();
		
		// containers
		else if (model instanceof AttributeListModel)
			part = new AttributeListEditPart();
		else if (model instanceof ParameterListModel)
			part = new ParameterListEditPart();
		else if (model instanceof AxiomContentsModel)
			part = new AxiomContentsEditPart();
		
		// connections
		else if (model instanceof ArrowConnectionModel)
			part = new ArrowConnectionEditPart();
		else if (model instanceof ManhattanConnectionModel)
			part = new ManhattanConnectionEditPart();
		else 
			Messages.showError("Error in AxiomEditPartFactory.\n" +
					"AxiomEditor doesn't know how to handle: " 
					+ model.getClass().toString());

		part.setModel(model);
		
		// Auto select certain models.
		if (model instanceof MemberOfModel ||
				model instanceof RelationModel ||
				model instanceof OperatorModel) {
			context.getViewer().select(part);
		}
		
		return part;
	}

}
