package org.infrawebs.axiom.editor.editparts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.infrawebs.axiom.editor.editpolicies.DeleteRelationEditPolicy;
import org.infrawebs.axiom.editor.figures.RatioConnectionAnchor;
import org.infrawebs.axiom.editor.figures.RelationFigure;
import org.infrawebs.axiom.model.Compactable;
import org.infrawebs.axiom.model.ParameterListModel;
import org.infrawebs.axiom.model.RelationModel;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;

public class RelationEditPart extends EditPartWithConnections implements NodeEditPart
{
	@Override
	public void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new DeleteRelationEditPolicy());
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		RelationModel model = (RelationModel)getModel();
		if (evt.getPropertyName().equals(Compactable.ID_PROPERTY_COMPACT)) {
			// If the "compactness" of the model has just changed, change the figure.
			RelationFigure localFigure = (RelationFigure)getFigure();
			if (model.isCompact() != localFigure.isCompact()) {
				localFigure.setCompact(model.isCompact(), model.getConstraint());
			}
		}
		else {
			super.propertyChange(evt);
		}
	}
	
	@Override
	public IFigure createFigure() {
		RelationModel model = (RelationModel)getModel();
		String prefixedName = AxiomEditorPlugin.ontoStore.getPrefixedName(model.getRelation());
		return new RelationFigure(prefixedName, model.isCompact()); 
	}
	
	@Override
	public List getModelChildren() {
		RelationModel model = (RelationModel)getModel();
		// The only additional child is the parameter list.
		List<ParameterListModel> modelChildren = new ArrayList<ParameterListModel>();
		modelChildren.add(model.getRefinableList());
		return modelChildren;
	}
	
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		RelationModel model = (RelationModel)getModel();
		RelationFigure localFigure = (RelationFigure)getFigure();
		localFigure.setEnabled(model.isFixedInExpression());
	}
	
	@Override
	protected void refreshTargetConnections() {
		super.refreshTargetConnections();
		refreshVisuals();
	}
	
	public ParameterListEditPart getParameterList() {
		// Find the first (and only) parameter list among the child editparts.
		for (int i = 0; i < this.children.size(); i++) {
			if (this.children.get(i) instanceof ParameterListEditPart) {
				return (ParameterListEditPart)this.children.get(i);
			}
		}
		return null;
	}
	
	@Override
	public IFigure getContentPane() {
		// Why this panel???
		return ((RelationFigure)getFigure()).getBodyPanel();
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return new RatioConnectionAnchor(getFigure(), 0.3, 0.0);		
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return new RatioConnectionAnchor(getFigure(), 0.3, 0.0);
	}
	
}
