package org.infrawebs.axiom.editor.editpolicies;

import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.infrawebs.axiom.model.ConnectionModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.model.commands.ConnectionCreateCommand;
import org.infrawebs.axiom.model.commands.ConnectionReconnectCommand;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.axiom.operations.SemanticConsistence;

public class AxiomGraphicalNodeEditPolicy extends GraphicalNodeEditPolicy
{
	@Override
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		ModelWithConnections src = (ModelWithConnections)request.getSourceEditPart().getModel();
		ModelWithConnections dst = (ModelWithConnections)request.getTargetEditPart().getModel();
		if (!(SemanticConsistence.allowMoreChildren(src) && SemanticConsistence.allowMoreParents(dst) &&
				SemanticConsistence.allowConnection(src, dst))) {
			return null;
		}

		ConnectionCreateCommand connCommand = (ConnectionCreateCommand)request.getStartCommand();
		ConnectionModel conn = connCommand.getConnection();
		conn.setTarget(dst);
		return ModelOperations.addSpreadVarNameCommands(connCommand, src, dst);
	}

	@Override
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		ModelWithConnections src = (ModelWithConnections)getHost().getModel();
		if (!SemanticConsistence.allowMoreChildren(src)) {
			return null;
		}

		ConnectionModel conn = (ConnectionModel)request.getNewObject();
		conn.setSource(src);
		ConnectionCreateCommand command = new ConnectionCreateCommand(conn);
		request.setStartCommand(command);
		return command;
	}

	@Override
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		ConnectionModel connection = (ConnectionModel)request.getConnectionEditPart().getModel();
		ModelWithConnections src = connection.getSource();
		ModelWithConnections dst = (ModelWithConnections)request.getTarget().getModel();
		if (!(SemanticConsistence.allowMoreParents(dst) &&
				SemanticConsistence.allowConnection(src, dst))) {
			return null;
		}

		ConnectionReconnectCommand connCommand = new ConnectionReconnectCommand(connection);
		connCommand.setNewTarget(dst);
		return ModelOperations.addSpreadVarNameCommands(connCommand, src, dst);
	}

	@Override
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		ConnectionModel connection = (ConnectionModel)request.getConnectionEditPart().getModel();
		ModelWithConnections src = (ModelWithConnections)request.getTarget().getModel();
		ModelWithConnections dst = connection.getTarget();
		if (!(SemanticConsistence.allowMoreChildren(src) &&
				SemanticConsistence.allowConnection(src, dst))) {
			return null;
		}

		ConnectionReconnectCommand connCommand = new ConnectionReconnectCommand(connection);
		connCommand.setNewSource(src);
		// TODO: Mark the old connection disabled and skip it during DFS.
		return ModelOperations.addSpreadVarNameCommands(connCommand, src, dst);
	}

}
