package org.infrawebs.axiom.editor.editpolicies;

import org.eclipse.gef.*;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.*;
import org.eclipse.gef.editpolicies.*;
import org.eclipse.draw2d.*;
import java.util.*;
import org.eclipse.gef.editparts.*;
import org.eclipse.draw2d.geometry.*;

public abstract class ToolbarLayoutEditPolicy extends OrderedLayoutEditPolicy
{
	private IFigure layoutTargetFeedback;
	private static final int FEEDBACK_WIDTH = 4; 
	
	protected IFigure createLayoutTargetFeedback() {
		Shape f = new RectangleFigure();
		f.setBackgroundColor(ColorConstants.black);
		addFeedback(f);
		return f;
	}

	protected IFigure getLayoutTargetFeedback() {
		if(this.layoutTargetFeedback == null) {
			this.layoutTargetFeedback = createLayoutTargetFeedback();
		}
		return this.layoutTargetFeedback;
	}
	
	private ToolbarLayout getToolbarLayout() {
		AbstractGraphicalEditPart host = (AbstractGraphicalEditPart)getHost();
		LayoutManager manager = host.getContentPane().getLayoutManager();
		return (manager instanceof ToolbarLayout) ? (ToolbarLayout)manager : null; 
	}
	
	@Override
	protected void showLayoutTargetFeedback(Request request) {
		ToolbarLayout layout = getToolbarLayout();  
		if (layout == null) {
			return;
		}
		boolean verticalLayout = !layout.isHorizontal();
		IFigure l = getLayoutTargetFeedback();
		AbstractGraphicalEditPart after = (AbstractGraphicalEditPart)getInsertionReference(request);
		
		Point loc = null;
		Dimension size = null;
		
		if (after == null) {
			if(getHost().getChildren().size() == 0) {
				Figure contentPane = (Figure)((AbstractGraphicalEditPart)getHost()).getContentPane();
				int mid = verticalLayout? contentPane.getBounds().height/2+contentPane.getLocation().y
				                          : contentPane.getBounds().width/2+contentPane.getLocation().x;
				loc = verticalLayout? new Point(contentPane.getLocation().x, mid) : new Point(mid,contentPane.getLocation().y);
				size = verticalLayout? new Dimension(contentPane.getSize().width,FEEDBACK_WIDTH)
				                                 : new Dimension(FEEDBACK_WIDTH,contentPane.getSize().height);
			}
			else {
				int lastIndex = getHost().getChildren().size()-1;
				Figure last = (Figure)((GraphicalEditPart)getHost().getChildren().get(lastIndex)).getFigure();
				loc = last.getLocation();
				last.translateToAbsolute(loc);
				loc = verticalLayout ? loc.getTranslated(0,last.getSize().height)
						: loc.getTranslated(last.getSize().width,0);
				size = verticalLayout ? new Dimension(last.getBounds().width , FEEDBACK_WIDTH)
						: new Dimension(FEEDBACK_WIDTH,last.getBounds().height);
			}
		}
		else {
			int indexAfter = getHost().getChildren().indexOf(after);
			//IFigure f = after.getFigure();
			
			if (indexAfter == 0) {
				Figure first = (Figure)((GraphicalEditPart)getHost().getChildren().get(0)).getFigure();
				loc = first.getLocation();
				first.translateToAbsolute(loc);
				loc = verticalLayout? loc.getTranslated(0,-FEEDBACK_WIDTH):loc.getTranslated(-FEEDBACK_WIDTH,0);
				size = verticalLayout?
				  new Dimension(first.getBounds().width , FEEDBACK_WIDTH)
				  : new Dimension(FEEDBACK_WIDTH,first.getBounds().height);
			}
			else {
				Figure prec = (Figure)((GraphicalEditPart)getHost().getChildren().get(indexAfter-1)).getFigure();
				Figure next = (Figure)((GraphicalEditPart)getHost().getChildren().get(indexAfter)).getFigure();
				if (verticalLayout)
				{
					int minPrec = prec.getLocation().y+prec.getBounds().height;
					int maxNext = next.getLocation().y;
					int interval = minPrec - maxNext;
					int midInterval = minPrec + interval/2;
					loc = new Point(prec.getBounds().x, midInterval);
					prec.translateToAbsolute(loc);
					size = new Dimension(prec.getSize().width, FEEDBACK_WIDTH);
				}
				else {
					int maxPrec = prec.getLocation().x+prec.getBounds().width;
					int minNext = next.getLocation().x;
					int interval = minNext - maxPrec;
					int midInterval = maxPrec + interval/2;
					loc = new Point(midInterval, prec.getBounds().y);
					prec.translateToAbsolute(loc);
					size = new Dimension(FEEDBACK_WIDTH, prec.getSize().height);
				}
			}
		}

		l.setBounds(new Rectangle(loc, size));
	}

	@Override
	protected void eraseLayoutTargetFeedback(Request request) {
		if(this.layoutTargetFeedback != null) {
			removeFeedback(this.layoutTargetFeedback);
		}
		this.layoutTargetFeedback = null;
	}
		
	@Override
	protected EditPart getInsertionReference(Request request) {
		ToolbarLayout layout = getToolbarLayout();  
		if (layout == null) {
			return null;
		}

		EditPart insertion = null;
		
		boolean verticalLayout = !layout.isHorizontal();
		
		int location =
			verticalLayout ? ((DropRequest)request).getLocation().y : ((ChangeBoundsRequest)request).getLocation().x;

		List children = ((GraphicalEditPart)getHost()).getContentPane().getChildren();
		if (children.isEmpty()) {
			return null;
		}

		Iterator it = children.iterator();
		IFigure childFigure = null;
		int middleChildFigure = 0;
		while (it.hasNext()) {
			childFigure = (IFigure)it.next();
			middleChildFigure = verticalLayout
					? childFigure.getBounds().y + childFigure.getBounds().height/2
					: childFigure.getBounds().x + childFigure.getBounds().width/2;
			if (location < middleChildFigure) {
				break;
			}
		}
		if (location < middleChildFigure) {
			IFigure partFigure = childFigure;
			insertion = (EditPart)getHost().getViewer().getVisualPartMap().get(partFigure);
		}
		else {
			insertion = null;
		}
			
		//Il faut prendre le premier EditPart aprs insertion qui ne se trouve pas dans 
		//la liste des EditParts concerns par la requte.
		//En effet, que la requte soit de type add children ou move children, il faut s'assurer
		//que l'EditPart de rfrence pour l'insertion des childs
		//appartiendra encore  la liste des enfants de l'hte
		//au moment de l'excution de la commande. Si l'EditPart de rfrence appartenait
		// la liste des children  ajouter, il pourrait subir un orphan avant l'excution de la
		//commande. Il faut viter cela.
		EditPart after = null;
		if(insertion == null || ! (request instanceof GroupRequest) ) {
			after = insertion;
		}
		else {
			List requestChildren = ((GroupRequest)request).getEditParts();
			List hostChildren = getHost().getChildren();
			int indexAfter = hostChildren.indexOf(insertion);
			while(indexAfter<hostChildren.size() && requestChildren.contains(hostChildren.get(indexAfter)))
				++indexAfter;
			if(indexAfter == hostChildren.size())
				after = null;
			else
				after = (EditPart)hostChildren.get(indexAfter);
		}
		
		return after;			
	}
}
