package org.infrawebs.axiom.editor.figures;

import org.eclipse.draw2d.Border;
import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.infrawebs.axiom.model.Compactable;
import org.infrawebs.axiom.settings.ColorSettings;
import org.infrawebs.axiom.settings.FontSettings;

public abstract class MemberOfFigure extends Figure implements Compactable
{
	protected Border baseBorder = null; 
	protected Panel headerPanel = null;
	protected Panel bodyPanel = null;
	protected Label conceptLabel = null;
	protected VarNameFigure varNameFigure = null;
	private boolean isCompact = false;
	
	public MemberOfFigure(String conceptName,
			String variableName, boolean existsFlag, boolean isCompact) {
		// this
		this.baseBorder = new LineBorder(ColorConstants.darkGray, 1);
		this.setBorder(this.baseBorder);
		this.setBackgroundColor(ColorSettings.MEMBEROF_BODY_BACK);
		this.setOpaque(true);
		
			// headerPanel
			this.headerPanel = new Panel();
			BorderLayout headerLayout = new BorderLayout();
			this.headerPanel.setLayoutManager(headerLayout);
			this.headerPanel.setBorder(new PartialBorder(ColorConstants.black, new Insets(0, 0, 1, 0)));
			this.headerPanel.setOpaque(true);

				this.varNameFigure = new VarNameFigure(variableName, existsFlag);

				// conceptLabel
				this.conceptLabel = new Label();
				this.conceptLabel.setText(conceptName);
				this.conceptLabel.setForegroundColor(ColorSettings.MEMBEROF_HEADER_FORE);
				this.conceptLabel.setFont(FontSettings.MEMBEROF_HEADER);
				this.conceptLabel.setOpaque(true);
				
				this.headerPanel.add(this.conceptLabel, BorderLayout.RIGHT);
				this.headerPanel.add(this.varNameFigure, BorderLayout.LEFT);
		
			// bodyPanel
			this.bodyPanel = new Panel();
			ToolbarLayout bodyLayout = new ToolbarLayout(false);
			bodyLayout.setStretchMinorAxis(true);
			this.bodyPanel.setLayoutManager(bodyLayout);
			this.bodyPanel.setBackgroundColor(ColorSettings.MEMBEROF_BODY_BACK);
			this.bodyPanel.setOpaque(true);
	
		add(this.bodyPanel);
		add(this.headerPanel);

		setCompact(isCompact);
	}

	public boolean isCompact() {
		return this.isCompact;
	}

	public void setCompact(boolean isCompact) {
		this.isCompact = isCompact;
		if (isCompact) {
			// When in compact mode, align the header over the body.
			this.setLayoutManager(new StackLayout());
		}
		else {
			this.setLayoutManager(new BorderLayout());
			setConstraint(this.headerPanel, BorderLayout.TOP);
			setConstraint(this.bodyPanel, BorderLayout.CENTER);
		}
	}
	
	public void setCompact(boolean isCompact, Object constraint) {
		setCompact(isCompact);
		// Realign this figure in its container.
		if (constraint != null) {
			getParent().setConstraint(this, constraint);
		}
	}

	public Figure getDirectEditFigure() {
		return null;
	}

	public Panel getBodyPanel() {
		return this.bodyPanel;
	}

	public abstract Color getHeaderColorEnabled();
	public abstract Color getHeaderColorDisabled();
	public abstract Image getIconEnabled();
	public abstract Image getIconDisabled();

	@Override
	public void setEnabled(boolean enabled) {
		this.headerPanel.setBackgroundColor(
				enabled	? getHeaderColorEnabled() : getHeaderColorDisabled());
		this.conceptLabel.setIcon(enabled ? getIconEnabled() : getIconDisabled());
	}

	public void setConceptName(String conceptName) {
		this.conceptLabel.setText(conceptName);
	}

	public VarNameFigure getVarNameFigure() {
		return this.varNameFigure;
	}

}
