package org.infrawebs.axiom.editor.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.infrawebs.axiom.model.Compactable;
import org.infrawebs.axiom.settings.ColorSettings;
import org.infrawebs.axiom.settings.FontSettings;
import org.infrawebs.ontology.ui.OntoImages;

public class RelationFigure extends Figure implements Compactable {

	private Panel headerPanel = null;
	private Panel bodyPanel = null;
	private Label relationLabel = null;
	private boolean isCompact = false;
	
	public RelationFigure(String relationName, boolean isCompact) {
		// this
		this.setBorder(new LineBorder(ColorConstants.darkGray, 1));
		this.setBackgroundColor(ColorSettings.MEMBEROF_BODY_BACK);
		this.setOpaque(true);
		
			// headerPanel
			this.headerPanel = new Panel();
			BorderLayout headerLayout = new BorderLayout();
			this.headerPanel.setLayoutManager(headerLayout);
			this.headerPanel.setBorder(new PartialBorder(ColorConstants.black, new Insets(0, 0, 1, 0)));
			this.headerPanel.setOpaque(true);

				// relationLabel
				this.relationLabel = new Label(relationName);
				this.relationLabel.setForegroundColor(ColorSettings.RELATION_HEADER_FORE);
				this.relationLabel.setFont(FontSettings.RELATION_HEADER);
				this.relationLabel.setOpaque(true);
				
				this.headerPanel.add(this.relationLabel);
		
			// bodyPanel
			this.bodyPanel = new Panel();
			ToolbarLayout bodyLayout = new ToolbarLayout(false);
			bodyLayout.setStretchMinorAxis(true);
			this.bodyPanel.setLayoutManager(bodyLayout);
			this.bodyPanel.setBackgroundColor(ColorSettings.MEMBEROF_BODY_BACK);
			this.bodyPanel.setOpaque(true);
	
		this.add(this.bodyPanel);
		this.add(this.headerPanel);

		setCompact(isCompact);
	}
	
	public boolean isCompact() {
		return this.isCompact;
	}

	public void setCompact(boolean isCompact) {
		this.isCompact = isCompact;
		if (isCompact) {
			// When in compact mode, align the header over the body.
			this.setLayoutManager(new StackLayout());
		}
		else {
			this.setLayoutManager(new BorderLayout());
			setConstraint(this.headerPanel, BorderLayout.TOP);
			setConstraint(this.bodyPanel, BorderLayout.CENTER);
		}

		// When in compact mode, move the Relation icon to the top-left
		// corner so that it is visible.
		this.headerPanel.setConstraint(this.relationLabel,
				isCompact ? BorderLayout.LEFT : BorderLayout.CENTER);

		// Realign the list figure.
		if (getBodyPanel().getChildren().size() > 0) {
			Compactable listFigure = (Compactable)getBodyPanel().getChildren().get(0);
			listFigure.setCompact(isCompact);
		}
	}
	
	public void setCompact(boolean isCompact, Object constraint) {
		setCompact(isCompact);
		// Realign this figure in its container.
		if (constraint != null) {
			getParent().setConstraint(this, constraint);
		}
	}

	public Panel getBodyPanel() {
		return this.bodyPanel;
	}

	@Override
	public void setEnabled(boolean enabled) {
		this.relationLabel.setBackgroundColor(enabled
				? ColorSettings.RELATION_HEADER_BACK
				: ColorSettings.RELATION_HEADER_BACK_DISABLED);
		this.relationLabel.setIcon(OntoImages.getImage(enabled
				? OntoImages.RELATION
				: OntoImages.RELATION_DISABLED));
	}

}
