package org.infrawebs.axiom.logexpr;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.infrawebs.common.Messages;
import org.infrawebs.ontology.ui.OntoImages;

class LESelectionDialog extends SelectionDialog
{
	private List<LEShell> inputShells = null;
	private List<LEShell> outputShells = null;
	// A listbox of logical expression identifiers.
	private org.eclipse.swt.widgets.List list = null;  
	
	public LESelectionDialog(Shell parentShell, List<LEShell> leShells) {
		super(parentShell);
		this.inputShells = leShells;
		setShellStyle(this.getShellStyle() | SWT.RESIZE | SWT.MAX);
		setTitle("Select logical expressions to load...");
		setDefaultImage(OntoImages.getImage(OntoImages.AXIOM));
		this.setBlockOnOpen(true);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite)super.createDialogArea(parent);
		composite.setLayout(new FillLayout());
		this.list = new org.eclipse.swt.widgets.List(composite, SWT.BORDER);
		// Load the identifiers of the logical expression into the listbox.
		for (LEShell shell: this.inputShells) {
			this.list.add(shell.getIdentifier());
		}
		this.list.select(0);
		return composite;
	}

	@Override
	public boolean close() {
		this.list.dispose();
		return super.close();
	}

	@Override
	protected void okPressed() {
		int[] indices = this.list.getSelectionIndices();
		if (indices.length == 0) {
			Messages.showWarning("Please, select at least one logical expression.");
			return;
		}
		// Store selection.
		this.outputShells = new ArrayList<LEShell>(indices.length);
		for (int i = 0; i < indices.length; i++) {
			this.outputShells.add(this.inputShells.get(indices[i]));
		}
		super.okPressed();
	}

	@Override
	protected void cancelPressed() {
		this.outputShells = null;
		super.cancelPressed();
	}

	public List listSelected() {
		return this.outputShells;
	}
}
