package org.infrawebs.axiom.model;

import java.io.IOException;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.common.Util;
import org.infrawebs.ontology.RestorationException;
import org.infrawebs.ontology.shells.AttributeShell;
import org.infrawebs.ontology.shells.OntologyShell;
import org.omwg.ontology.Concept;
import org.wsmo.common.Identifier;

public class AttributeModel extends RefinableModel
{
	private static final long serialVersionUID = 200512242100L;

	public static final String ID_PROPERTY_ATTRIBUTE_INHERITANCE = "ID_PROPERTY_ATTRIBUTE_INHERITANCE";
	public static final String ID_PROPERTY_ATTRIBUTE_CARDINALITY = "ID_PROPERTY_ATTRIBUTE_CARDINALITY";

	public AttributeModel(AxiomContentsModel contents, AttributeShell attrInfo) {
		super(contents, attrInfo);
	}

	/**
	 * The default variable name is based on the attrbute's name.
	 */
	@Override
	public VarNameModel createVarNameModel() {
		String prefix = Util.getLocalName(this.refinable);
		return new VarNameModel(this.contents.getVarNameList(), prefix);
	}

	@Override
	public IPropertyDescriptor[] createPropertyDescriptors() {
		IPropertyDescriptor[] superDescriptors = super.createPropertyDescriptors();
		int oldLen = superDescriptors.length;
		IPropertyDescriptor[] descriptors = new IPropertyDescriptor[oldLen + 2];
		System.arraycopy(superDescriptors, 0, descriptors, 0, oldLen);
		descriptors[oldLen + 0] = new PropertyDescriptor(ID_PROPERTY_ATTRIBUTE_INHERITANCE, "Inheritance");
		descriptors[oldLen + 1] = new PropertyDescriptor(ID_PROPERTY_ATTRIBUTE_CARDINALITY, "Cardinality");
		return descriptors;
	}
	
	@Override
	public Object getPropertyValue(Object propName) {
		AttributeShell attrShell = (AttributeShell)getRefinable();
		
		if (propName.equals(ID_PROPERTY_ATTRIBUTE_INHERITANCE)) {
			return attrShell.getTypeAsString();
		}
		else if (propName.equals(ID_PROPERTY_ATTRIBUTE_CARDINALITY)) {
			return attrShell.getCardinalityAsString();
		}
		else {
			return super.getPropertyValue(propName);
		}
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
		// Do not write the attribute shell itself. Instead, write all it's fields manually.
		// Replace ontology objects with their identifiers.
		AttributeShell attrShell = (AttributeShell)getRefinable(); 
		out.writeObject(attrShell.getConcept().getIdentifier());
		out.writeObject(attrShell.getAttribute().getIdentifier());
		out.writeInt(attrShell.getInheritanceType());
	}

	private void readObject(java.io.ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		// Read the elements that were saved for the attribute shell.
		Identifier conceptId = (Identifier)in.readObject();
		Identifier attributeId = (Identifier)in.readObject();
		int inheritanceType = in.readInt(); 

		// Restore the definition of the attributes's concept.
		Concept concept = (Concept)AxiomEditorPlugin.ontoStore.findDefinition(
				conceptId, Concept.class);
		if (concept == null) {
			throw new RestorationException("Concept " + conceptId + " not found\n" +
					"while loading attribute " + attributeId + ".");
		}

		// Use the restored concept and attribute identifier to find the attribute object.
		for (AttributeShell attrShell: OntologyShell.listAttributesRecursive(concept)) {
			if (attributeId.equals(attrShell.getIdentifier()) &&
					(concept == attrShell.getConcept()) &&
					(inheritanceType == attrShell.getInheritanceType())) {
		 		// Reconstruct the attribute shell.
				this.refinable = new AttributeShell(attrShell.getAttribute(), concept, inheritanceType);
				return; 
			}
		}
		throw new RestorationException("Invalid attribute " +
				attributeId + " for concept " + conceptId + ".");
	}

}
