package org.infrawebs.axiom.model;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.omwg.logicalexpression.Constants;

public class AxiomRootModel extends ModelWithConnections
{
	public static final String ID_PROPERTY_AXIOM_NAME = "ID_PROPERTY_AXIOM_NAME";
	public static final String ID_PROPERTY_AXIOM_DESCRIPTION = "ID_PROPERTY_AXIOM_DESCRIPTION";

	private static final long serialVersionUID = 200512242100L;
	private static final String DC_DESCRIPTION = "http://purl.org/dc/elements/1.1" + Constants.SQNAME_DEL + "description";

	private String axiomName = null;
	private Map<String, String> nfps = new HashMap<String, String>();

	public AxiomRootModel(AxiomContentsModel contents, String axiomName) {
		super(contents);
		if (axiomName == null || axiomName.length() == 0) {
			throw new IllegalArgumentException("Axiom name cannot be empty.");
		}
		this.axiomName = axiomName;
		setPropertyValue(ID_PROPERTY_AXIOM_DESCRIPTION, "Auto-generated axiom by Axiom Editor");
		this.setConstraint(new Rectangle(10, 10, 50, 40));
	}
	
	public String getAxiomName() {
		return this.axiomName;
	}

	public Map getNFPs() {
		return this.nfps; 
	}

	// The AxiomRootModel should point to exactly 1 model which is the top of the expression tree. 
	public ModelWithConnections getTopModel() {
		List<ModelWithConnections> children = listGrafChildren();
		if (children.size() > 0) {
			return children.get(0); 
		}
		return null;
	}

	@Override
	public IPropertyDescriptor[] createPropertyDescriptors() {
		IPropertyDescriptor[] descriptors = new IPropertyDescriptor[2];
		descriptors[0] = new TextPropertyDescriptor(ID_PROPERTY_AXIOM_NAME, "Name");
		descriptors[1] = new TextPropertyDescriptor(ID_PROPERTY_AXIOM_DESCRIPTION, "Description");
		return descriptors;
	}

	@Override
	public Object getPropertyValue(Object id) {
		if (id.equals(ID_PROPERTY_AXIOM_NAME)) {
			return this.axiomName;
		}
		else if (id.equals(ID_PROPERTY_AXIOM_DESCRIPTION)) {
			return this.nfps.get(DC_DESCRIPTION).toString();
		}
		return null;
	}

	@Override
	public void setPropertyValue(Object id, Object value) {
		if (id.equals(ID_PROPERTY_AXIOM_NAME)) {
			this.axiomName = value.toString();
		}
		else if (id.equals(ID_PROPERTY_AXIOM_DESCRIPTION)) {
			this.nfps.put(DC_DESCRIPTION, (String)value);
		}
	}

	@Override
	public void setConstraint(Object constraint) {
		Rectangle newRect = (Rectangle)constraint;
		// Keep original size.
		Rectangle oldRect = (Rectangle)getConstraint();
		if (oldRect != null) {
			newRect.setSize(oldRect.getSize());
		}
		super.setConstraint(newRect);
	}

	/**
	 * The depth of the axiom root element is always zero.
	 */
	@Override
	protected int findGrafDepth(Set<ModelWithConnections> visited) {
		if (!visited.contains(this)) {
			visited.add(this);
		}
		return 0;
	}
	
}
