package org.infrawebs.axiom.model;

import java.io.IOException;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.common.Util;
import org.infrawebs.ontology.RestorationException;
import org.omwg.ontology.Concept;
import org.omwg.ontology.Instance;
import org.wsmo.common.Identifier;

public class InstanceModel extends MemberOfModel
{
	private static final long serialVersionUID = 200512242100L;

	public static final String ID_PROPERTY_VALUE = "ID_PROPERTY_VALUE";

	private transient Instance instance = null;

	public InstanceModel(AxiomContentsModel contents, Instance instance, Concept parent) {
		super(contents, parent);
		this.setInstance(instance);
	}

	public String getValue() {
		return Util.getLocalName(this.instance);
	}

	@Override
	public IPropertyDescriptor[] createPropertyDescriptors() {
		IPropertyDescriptor[] superDescriptors = super.createPropertyDescriptors();
		int oldLen = superDescriptors.length;
		IPropertyDescriptor[] descriptors = new IPropertyDescriptor[oldLen + 1];
		System.arraycopy(superDescriptors, 0, descriptors, 0, oldLen);
		descriptors[oldLen + 0] = new PropertyDescriptor(ID_PROPERTY_VALUE, "Instance");
		return descriptors;
	}

	@Override
	public Object getPropertyValue(Object id) {
		if (id.equals(ID_PROPERTY_VALUE)) {
			return getValue();
		}
		return super.getPropertyValue(id);
	}


	public Instance getInstance() {
		return this.instance;
	}
	
	public void setInstance(Instance instance) {
		this.instance = instance;
	}
	
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
		// Write just the instance identifier, not the instance itself.
		out.writeObject(this.instance.getIdentifier());
	}

	private void readObject(java.io.ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		// Read the instance identifier.
		Identifier instanceId = (Identifier)in.readObject();

		// Restore the definition of the instance.
		this.instance = (Instance)AxiomEditorPlugin.ontoStore.findDefinition(
				instanceId, Instance.class);
		if (this.instance == null) {
			throw new RestorationException("Instance " + instanceId + " not found.");
		}
	}

}
