package org.infrawebs.axiom.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ParameterListModel extends RefinableListModel
{
	private static final long serialVersionUID = 200510291900L;

	public ParameterListModel(RelationModel owner) {
		super(owner);
	}
	
	/**
	 * All parameters should be refined by exactly 1 subexpression.
	 */
	@Override
	public List<ModelWithConnections> listRefiningModels() {
		List<ModelWithConnections> result = new ArrayList<ModelWithConnections>();
		for (int i = 0; i < getChildren().size(); i++) {
			ParameterModel paramModel = (ParameterModel)getChildren().get(i);
			ModelWithConnections refinement = paramModel.getRefiningModel();
			// If a parameter is not refined, use the very parameter as a dummy refinement.
			if (refinement == null) {
				refinement = paramModel;
			}
			result.add(refinement);
		}
		return result;
	}

	/**
	 * Finds the element after which the owner relation must be included in the expression tree.
	 * This should be one of the deepest elements which refines one of the parameters.
	 * @return The first refined parameter whose refining model is as deep as possible
	 * in the graph.
	 */
	public ParameterModel getDeepestParameter() {
		int maxDepth = -1;
		ParameterModel result = null;
		Set<ModelWithConnections> visited = new HashSet<ModelWithConnections>();
		for (int i = 0; i < getChildren().size(); i++) {
			ParameterModel param = (ParameterModel)getChildren().get(i);
			ModelWithConnections refiningModel = param.getRefiningModel(); 
			if (refiningModel != null) {
				// Do not search for the depth through the owner relation but find alternate ways.
				visited.clear();
				visited.add(getOwner());
				int depth = refiningModel.findGrafDepth(visited);
				if (depth > maxDepth) {
					maxDepth = depth;
					result = param;
				}
			}
		}
		return result;
	}

}
