package org.infrawebs.axiom.outline.editparts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.EditPolicy;
import org.infrawebs.axiom.editor.editpolicies.DeleteRelationEditPolicy;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.model.ParameterModel;
import org.infrawebs.axiom.model.RelationModel;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.ontology.ui.OntoImages;

public class RelationTreeEditPart extends TreeEditPartWithConnections
{
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new DeleteRelationEditPolicy());
	}

	@Override
	protected void refreshVisuals() {
		RelationModel model = (RelationModel)getModel();
		setWidgetText(AxiomEditorPlugin.ontoStore.getPrefixedName(model.getRelation())); // "Relation: " + 
		setWidgetImage(OntoImages.getImage(OntoImages.RELATION));
	}

	protected List getModelChildren_old() {
		RelationModel relation = (RelationModel)getModel();
		List deeperChildren = getDeeperModelChildren(
				relation.getRefinableList().listRefiningModels());
		return deeperChildren;
	}

	@Override
	protected List getModelChildren() {
		RelationModel relation = (RelationModel)getModel();
		if (relation.isFixedInExpression()) {
			return relation.getRefinableList().listRefiningModels();
		}

		// This is a floating relation. Its position has already been determined by the
		// "deepest parameter" - the parent of that relation in the outline tree.
		// Avoid recursion by replacing all refining models which are not deeper with
		// their respective parameter models.
		int relationDepth = relation.getGrafDepth();
		List<ModelWithConnections> childModels = new ArrayList<ModelWithConnections>();
		for (ModelWithConnections child: relation.getRefinableList().getChildren()) {
			ParameterModel param = (ParameterModel)child;
			ModelWithConnections refiningModel = param.getRefiningModel();
			if (refiningModel != null && refiningModel.getGrafDepth() > relationDepth) {
				childModels.add(refiningModel);
			}
			else {
				childModels.add(param);
			}
		}
		return childModels; 
	}
	
}
