package org.infrawebs.axiom.plugin;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;

import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ContributionItemFactory;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;

/**
 * Adds actions to a workbench window.
 */
public final class AxiomActionBarAdvisor extends ActionBarAdvisor
{
	public AxiomActionBarAdvisor(IActionBarConfigurer configurer) {
		super(configurer);
	}

	@Override
    protected void makeActions(IWorkbenchWindow window) {
    	super.makeActions(window);
		register(ActionFactory.SAVE.create(window));
		register(ActionFactory.SAVE_AS.create(window));
		register(ActionFactory.SAVE_ALL.create(window));
		register(ActionFactory.UNDO.create(window));
		register(ActionFactory.REDO.create(window));
		register(ActionFactory.CUT.create(window));
		register(ActionFactory.COPY.create(window));
		register(ActionFactory.PASTE.create(window));
		register(ActionFactory.DELETE.create(window));
		register(ActionFactory.SELECT_ALL.create(window));
		register(ActionFactory.FIND.create(window));
		register(ActionFactory.CLOSE.create(window));
		register(ActionFactory.CLOSE_ALL.create(window));
		register(ActionFactory.REVERT.create(window));
		register(ActionFactory.QUIT.create(window));
	}

	@Override
    protected void fillCoolBar(ICoolBarManager coolBar) {
		// TODO .............................. PI ...................................
		// ne raboti naredbata kakto trjabva
		
		coolBar.add(new Separator(IWorkbenchActionConstants.NEW_EXT));
		coolBar.add(new Separator(IWorkbenchActionConstants.OPEN_EXT));

		IToolBarManager fileToolBar = new ToolBarManager(coolBar.getStyle());
		fileToolBar.add(new Separator(IWorkbenchActionConstants.SAVE_GROUP));
		fileToolBar.add(getAction(ActionFactory.SAVE.getId()));
		fileToolBar.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		
		// Add to the cool bar manager
		coolBar.add(new ToolBarContributionItem(fileToolBar, IWorkbenchActionConstants.TOOLBAR_FILE));
		
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_EDITOR));
	}
	
	@Override
    protected void fillMenuBar(IMenuManager menuBar) {
		IWorkbenchWindow window = getActionBarConfigurer().getWindowConfigurer().getWindow();
		menuBar.add(createFileMenu());
		menuBar.add(createEditMenu());
		menuBar.add(createWindowMenu(window));
		menuBar.add(createHelpMenu(window));
		menuBar.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/**
	 * Creates and returns the File menu.
	 */
	private MenuManager createFileMenu() {
		MenuManager menu = new MenuManager("&File", IWorkbenchActionConstants.M_FILE); //$NON-NLS-1$

		menu.add(new GroupMarker(IWorkbenchActionConstants.FILE_START));
		menu.add(new GroupMarker(IWorkbenchActionConstants.NEW_EXT));
		menu.add(new GroupMarker(IWorkbenchActionConstants.OPEN_EXT));
		
		menu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		menu.add(getAction(ActionFactory.CLOSE.getId()));
		menu.add(getAction(ActionFactory.CLOSE_ALL.getId()));
		menu.add(new GroupMarker(IWorkbenchActionConstants.CLOSE_EXT));
		menu.add(new Separator());
		menu.add(getAction(ActionFactory.SAVE.getId()));
		menu.add(getAction(ActionFactory.SAVE_AS.getId()));
		menu.add(getAction(ActionFactory.SAVE_ALL.getId()));

		menu.add(getAction(ActionFactory.REVERT.getId()));
		IWorkbenchWindow window = getActionBarConfigurer().getWindowConfigurer().getWindow();
		menu.add(ContributionItemFactory.REOPEN_EDITORS.create(window));
		menu.add(new GroupMarker(IWorkbenchActionConstants.MRU));
		menu.add(new Separator());
		menu.add(getAction(ActionFactory.QUIT.getId()));
		menu.add(new GroupMarker(IWorkbenchActionConstants.FILE_END));
		return menu;
	}

	/**
	 * Creates and returns the Edit menu.
	 */
	private MenuManager createEditMenu() {
		MenuManager menu = new MenuManager("&Edit", IWorkbenchActionConstants.M_EDIT); //$NON-NLS-1$
		menu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_START));

		menu.add(getAction(ActionFactory.UNDO.getId()));
		menu.add(getAction(ActionFactory.REDO.getId()));
		menu.add(new GroupMarker(IWorkbenchActionConstants.UNDO_EXT));
		menu.add(new Separator());

		menu.add(getAction(ActionFactory.CUT.getId()));
		menu.add(getAction(ActionFactory.COPY.getId()));
		menu.add(getAction(ActionFactory.PASTE.getId()));
		menu.add(new GroupMarker(IWorkbenchActionConstants.CUT_EXT));

		menu.add(new Separator());
		menu.add(getAction(ActionFactory.DELETE.getId()));
		menu.add(getAction(ActionFactory.SELECT_ALL.getId()));
		menu.add(new Separator());

		menu.add(getAction(ActionFactory.FIND.getId()));
		menu.add(new GroupMarker(IWorkbenchActionConstants.FIND_EXT));
		menu.add(new Separator());
		menu.add(new GroupMarker(IWorkbenchActionConstants.ADD_EXT));

		menu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_END));
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		return menu;
	}

	private MenuManager createWindowMenu(IWorkbenchWindow window) {
		MenuManager menu = new MenuManager(AxiomEditorPlugin.getResourceString(
				"Window"), //$NON-NLS-1$
				IWorkbenchActionConstants.M_WINDOW);

		menu.add(ActionFactory.OPEN_NEW_WINDOW.create(window));

		menu.add(new Separator());
		MenuManager perspectiveMenu = new MenuManager(AxiomEditorPlugin.getResourceString(
				"Open_Perspective"), "openPerspective"); //$NON-NLS-1$ //$NON-NLS-2$
		IContributionItem perspectiveList = ContributionItemFactory.PERSPECTIVES_SHORTLIST.create(window);
		perspectiveMenu.add(perspectiveList);
		menu.add(perspectiveMenu);

		MenuManager viewMenu = new MenuManager(AxiomEditorPlugin.getResourceString(
				"Show_View")); //$NON-NLS-1$
		IContributionItem viewList = ContributionItemFactory.VIEWS_SHORTLIST.create(window);
		viewMenu.add(viewList);
		menu.add(viewMenu);

		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

		menu.add(ActionFactory.PREFERENCES.create(window));

		menu.add(ContributionItemFactory.OPEN_WINDOWS.create(window));

		return menu;
	}

	private MenuManager createHelpMenu(IWorkbenchWindow window) {
		MenuManager menu = new MenuManager(AxiomEditorPlugin.getResourceString(
				"Help"), IWorkbenchActionConstants.M_HELP); //$NON-NLS-1$
		// Welcome or intro page would go here
		menu.add(ActionFactory.HELP_CONTENTS.create(window));
		// Tips and tricks page would go here
		menu.add(new GroupMarker(IWorkbenchActionConstants.HELP_START));
		menu.add(new GroupMarker(IWorkbenchActionConstants.HELP_END));
		menu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		// About should always be at the bottom
		// To use the real RCP About dialog uncomment these lines
		// menu.add(new Separator());
		// menu.add(ActionFactory.ABOUT.create(window));

		return menu;
	}

}
