package org.infrawebs.axiom.plugin;

import java.util.Iterator;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;
import org.infrawebs.axiom.model.AxiomContentsModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.common.Messages;
import org.infrawebs.ontology.OntoLoader;
import org.infrawebs.ontology.OntoStore;
import org.infrawebs.ontology.shells.OntologyShell;
import org.infrawebs.ontology.shells.RefinableShell;
import org.infrawebs.ontology.ui.OntoPane;
import org.infrawebs.ontology.ui.OntoPanes;
import org.infrawebs.ontology.ui.OntoView;
import org.omwg.ontology.Concept;
import org.omwg.ontology.OntologyElement;
import org.omwg.ontology.Type;

/**
 * This OntoView uses the plug-in's shared OntoStore and handles double-clicks.
 * @see AxiomOntoPanes
 */
public class AxiomOntoView extends OntoView
{
	public static final String VIEW_ID = "org.infrawebs.axiom.plugin.axiomOntoView"; 

	@Override
	public OntoLoader getOntoLoader() {
		return AxiomEditorPlugin.ontoLoader;
	}

	@Override
	public OntoStore getOntoStore() {
		return AxiomEditorPlugin.ontoStore;
	}

	@Override
	protected OntoPanes createOntoPanes(Composite parent) {
		return new AxiomOntoPanes(parent);
	}

}

/**
 * Instances of this class handle mouse double-clicks.
 * @see OntoTreeMouseListener
 */
class AxiomOntoPanes extends OntoPanes
{
	public AxiomOntoPanes(Composite parent) {
		super(parent);
	}

	@Override
	public OntoPane createPane(OntologyShell ontology) {
		OntoPane ontoPane = super.createPane(ontology);
		if (ontoPane != null) {
			ontoPane.getTree().addMouseListener(new OntoTreeMouseListener(ontoPane));
		}
		return ontoPane;
	}
}

/**
 * Handles mouse double-clicks to create axiom elements or navigate in the ontology store.
 */
class OntoTreeMouseListener implements MouseListener 
{
	private OntoPane ontoPane = null;

	public OntoTreeMouseListener(OntoPane ontoPane) {
		this.ontoPane = ontoPane;
	}

	public void mouseDoubleClick(MouseEvent e) {
		Object item = this.ontoPane.getSelection();
		if (item == null) {
			return;
		}
		// Locate the type of attributes and parameters.
		else if (item instanceof RefinableShell) {
			Type type = ((RefinableShell)item).getFirstType();
			Concept concept = AxiomEditorPlugin.ontoStore.liftTypeToConcept(type);
			if (concept != null) {
				AxiomEditorPlugin.ontoLoader.jumpToDefinition(concept);
			}
		}
		// Create new axiom elements from local ontology elements.
		else if (item instanceof OntologyElement) {
			OntologyElement element = (OntologyElement)item;
			
			// Locate the definition of the element (possibly in an external file).
			OntologyElement definition = AxiomEditorPlugin.ontoLoader.jumpToDefinition(element);
			if (definition != null) {
				if (definition instanceof Concept) {
					demandSuperConcepts((Concept)definition);
				}
				
				AxiomContentsModel contents = AxiomEditorPlugin.getAxiomContentsModel(); 
				if (contents != null) {
					ModelWithConnections model = contents.getFactory().createAxiomElement(definition);
					ModelOperations.createAndConnectToAxiomRoot(model);
				}
			}
		}
	}

	private void demandSuperConcepts(Concept concept) {
		boolean loadAllOntologies = false;
		
		Iterator i = concept.listSuperConcepts().iterator();
		while (i.hasNext()) {
			Concept superConcept = (Concept)i.next();
			
			OntologyElement definition = AxiomEditorPlugin.ontoStore.findDefinition(superConcept);
			if (definition == null) {
				if (!loadAllOntologies) {
					loadAllOntologies = Messages.showQuestion("The concept you have chosen may have additional attributes.\n\n"
						+ "They may be inherited from a super-concept defined in another ontology which is not loaded.\n\n"
						+ "Do you want all additional inherited attributes to be shown too?");
					if (!loadAllOntologies)
						return;
				}
				// Locate the definition of the element (possibly in an external file).
				AxiomEditorPlugin.ontoLoader.jumpToDefinition(superConcept);				
//				loadOntologiesFromFile(fileName, false)) {
			}
		}
	}
	
	public void mouseDown(MouseEvent e) {
		// Do nothing on mouse down.
	}

	public void mouseUp(MouseEvent e) {
		// Do nothing on mouse up.
	}

}
