package org.infrawebs.axiom.plugin.actions;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.infrawebs.axiom.editor.inputs.PathEditorInput;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.axiom.plugin.extras.SimpleEditor;

public class OpenAction extends Action implements IWorkbenchWindowActionDelegate
{
	private IWorkbenchWindow window;

	public void init(IWorkbenchWindow workbenchWindow) {
		this.window = workbenchWindow;
	}

	public void dispose() {
		this.window = null;
	}

	public void selectionChanged(IAction action, ISelection selection) {
		// Obligation to implement.
	}

	public void run(IAction action) {
		// Show a file open dialog and let the user select a file.
		String fileName = getOpenDialog().open();
		if (fileName == null || fileName.length() == 0) {
			return;
		}

		IEditorRegistry editorRegistry = this.window.getWorkbench().getEditorRegistry();
		// The file extension determines the type of the editor.
		IEditorDescriptor descriptor = editorRegistry.getDefaultEditor(fileName);
		String editorId = (descriptor != null) ? descriptor.getId() : SimpleEditor.EDITOR_ID;
		IWorkbenchPage page = this.window.getActivePage();
		IEditorInput input = new PathEditorInput(new Path(fileName));
		try {
			// Create or focus an appropriate editor for the input specified.
			page.openEditor(input, editorId);
		}
		catch (PartInitException e) {
			e.printStackTrace();
		}
	}

	private static FileDialog openDialog = null;

    private static FileDialog getOpenDialog() {
    	if (openDialog == null) {
			openDialog = new FileDialog(AxiomEditorPlugin.getDefault().getWorkbench().
					getActiveWorkbenchWindow().getShell(), SWT.OPEN);
			openDialog.setText("Open file...");
			openDialog.setFilterNames(new String[] {
					"All supported types (*.axiom, *.wsml)",
					"Axioms (*.axiom)",
					"Ontologies (*.wsml)",
					"All files (*.*)"
			});
			openDialog.setFilterExtensions(new String[] {
					"*.axiom;*.wsml", //$NON-NLS-1$
					"*.axiom", //$NON-NLS-1$
					"*.wsml", //$NON-NLS-1$
					"*.*" //$NON-NLS-1$
			});
    	}
		return openDialog;
    }
    
}
