package org.infrawebs.axiom.text;

import java.util.Collection;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.parts.AbstractEditPartViewer;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.MessagePage;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.PageBookView;
import org.infrawebs.axiom.editor.AxiomEditor;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.axiom.text.actions.SaveAxiomTextAction;
import org.infrawebs.axiom.text.editparts.AxiomTextContentsEditPart;
import org.infrawebs.axiom.text.editparts.AxiomTextRootEditPart;

/**
 * A multi-page view containing the text representation of all axioms being edited.
 */
public class AxiomTextView extends PageBookView
{
	public static final String VIEW_ID = "org.infrawebs.axiom.text.axiomTextView";
	/**
	 * A single page containing the axom text from a single AxiomEditor.
	 */
	public static class OutlinePage extends ContentOutlinePage
	{
		private AxiomEditor editor = null;

		public OutlinePage(AxiomEditor editor) {
			super(new Viewer());
			this.editor = editor;
		}

		@Override
		public void createControl(Composite parent) {
			super.createControl(parent);
			Viewer viewer = (Viewer)getViewer();
			EditDomain editDomain = (EditDomain)this.editor.getAdapter(EditDomain.class);
    		viewer.setEditDomain(editDomain);
    		EditPart contents = new AxiomTextContentsEditPart(this.editor.getContents());
    		viewer.setContents(contents);
	    }

		/**
		 * @return The WSML text contained in the viewer control.
		 */
		public String getText() {
			String result = null;
			Viewer localViewer = (Viewer)getViewer();
			if (localViewer != null) {
				Text textControl = (Text)localViewer.getControl(); 
				if (textControl != null) {
					result = textControl.getText();
				}
			}
			return result;
		}
	}

	/**
	 * A viewer wrapping an SWT Text widget.
	 */
	private static class Viewer extends AbstractEditPartViewer
	{
		public Viewer() {
			setRootEditPart(new AxiomTextRootEditPart());
		}
		
		@Override
		public Control createControl(Composite parent) {
			Text text = new Text(parent, SWT.MULTI | SWT.WRAP | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER | SWT.READ_ONLY);
			text.setBackground(ColorConstants.listBackground);
			setControl(text);
			return text;
		}

		public EditPart findObjectAtExcluding(Point pt, Collection exclude, Conditional condition) {
			return null;
		}
		
	}

	@Override
	protected IPage createDefaultPage(PageBook book) {
		MessagePage page = new MessagePage();
		initPage(page);
		page.createControl(book);
		page.setMessage("Axiom text not available");
		return page;
	}

	@Override
	protected PageRec doCreatePage(IWorkbenchPart part) {
		if (!(part instanceof AxiomEditor)) {
			return null;
		}
		OutlinePage page = new OutlinePage((AxiomEditor)part);
		initPage(page);
		page.createControl(getPageBook());
		return new PageRec(part, page);
	}

	@Override
	protected void doDestroyPage(IWorkbenchPart part, PageRec pageRecord) {
		pageRecord.page.dispose();
		pageRecord.dispose();
	}

	@Override
	protected IWorkbenchPart getBootstrapPart() {
		return AxiomEditorPlugin.getAxiomEditor();
	}

	@Override
	protected boolean isImportant(IWorkbenchPart part) {
		return part instanceof IEditorPart;
	}

	@Override
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		
		IToolBarManager toolbar = site.getActionBars().getToolBarManager();
		toolbar.add(new SaveAxiomTextAction(this));
		
		site.getActionBars().updateActionBars();		
	}
	
}
