package org.infrawebs.axiom.text.editparts;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.swt.widgets.Text;
import org.infrawebs.axiom.editor.AxiomEditorContents;
import org.infrawebs.common.Util;

/**
 * This EditPart rebuilds the text of its text viewer every time
 * a command is executed in the associated editor. 
 */
public class AxiomTextContentsEditPart extends AbstractEditPart implements Observer
{
	private AxiomEditorContents editorContents = null;
	
	public AxiomTextContentsEditPart(AxiomEditorContents editorContents) {
		//setModel(editor.getContents().getAxiomContentsModel());
		this.editorContents = editorContents;
	}

	@Override
	public void activate() {
		super.activate();
		this.editorContents.addObserver(this);
	}

	@Override
	public void deactivate() {
		this.editorContents.deleteObserver(this);
		super.deactivate();
	}

	public void update(Observable o, Object arg) {
		// Regenerate the text on every change.
		refreshVisuals();
	}

	// Rebuilds the axiom text and updates the text viewer.
	@Override
	protected void refreshVisuals() {
		if (getViewer() == null) {
			return;
		}

		String text = Util.serialize(this.editorContents.createEnclosingWebService());
		Text control = (Text)getViewer().getControl();
		control.setText(text);
	}

	public DragTracker getDragTracker(Request request) {
		return null;
	}

	@Override
	protected void createEditPolicies() {
		// No edit policies to craate.
	}

	@Override
	protected void addChildVisual(EditPart child, int index) {
		// Obligation to implement.
	}

	@Override
	protected void removeChildVisual(EditPart child) {
		// Obligation to implement.
	}

}
