package org.infrawebs.ontology;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.common.Messages;
import org.infrawebs.common.Util;
import org.infrawebs.ontology.shells.OntologyShell;
import org.omwg.ontology.Ontology;
import org.omwg.ontology.OntologyElement;
import org.wsmo.common.Identifier;
import org.wsmo.common.TopEntity;
import org.wsmostudio.runtime.WSMORuntime;

public class OntoLoader
{
	private OntoStore ontoStore = null;
	// TODO: Use the InternetLoader!
//	private InternetLoader internetLoader = new InternetLoader(this);

	public OntoLoader(OntoStore ontoStore) {
		this.ontoStore = ontoStore;
	}

	public boolean loadOntologiesFromFile(String fileName, boolean silent) {
        try {
            File srcFile = new File(fileName);
            if (false == srcFile.exists()) {
                URL fileURL = AxiomEditorPlugin.getDefault().getBundle().getEntry("/");
                IPath path = new Path(Platform.asLocalURL(fileURL).getFile());
                fileName = path.toFile().getAbsolutePath() + "/" + fileName;
            }

            TopEntity[] tops = Util.parseFile(fileName);
			boolean foundOne = false;
			for (int i = 0; i < tops.length; i++) {
				if (tops[i] instanceof Ontology) {
					foundOne = true;
					Identifier newIdentifier = tops[i].getIdentifier();
					// Prevent multiple instances of the same ontology.
					if (this.ontoStore.getOntology(newIdentifier) == null) {
						this.ontoStore.addOntology(new OntologyShell((Ontology)tops[i]));
					} else {
						if (!silent) {
							Messages.showInformation("Ontology \"" + newIdentifier
									+ "\" already loaded.");
						}
					}
				}
			}
			if (!foundOne && !silent) {
				Messages.showWarning("File does not contain ontologies:\n"
						+ fileName);
			}
			return foundOne;
		}
		catch (Exception ex) {
			Messages.showError("Error loading ontologies from file " + fileName + "\n\n" +
					ex.getClass().getName() +
					"\nMessage: " + ex.getMessage() +
					"\nCause: " + ex.getCause());
			return false;
		}
	}

	/**
	 * @see OntoStore.findDefinition
	 * Extends OntoStore.findDefinition by trying to load the necessary ontology
	 * if the definition is not found in the ontoStore.
	 * @return The definition or null if it was not found anywhere.
	 */
	public OntologyElement findDefinition(
			Identifier elementId,
			Class elementClass,
			List<Ontology> importedOntos)
			throws RestorationException
	{
		OntologyElement element = this.ontoStore.findDefinition(elementId, elementClass);
		if (element != null) {
			return element;  // Ontology already loaded.
		}

		Ontology result = WSMORuntime.getIOManager().locateEntityContainer(
				elementId, elementClass, importedOntos, true);

		if (result == null) {
//			if (Messages.showConfirm("Search ontology \"" + ns + "\"\non the Internet?")) {
//				if (this.internetLoader.loadOntologies(ns)) {
//					...
//				}
//			}
			throw new RestorationException("The ontology defining\n" + elementId +
					"\nwas not found in the workspace.");
		}

		Identifier ontologyId = result.getIdentifier();
		if (this.ontoStore.getOntology(ontologyId) != null) {
			throw new RestorationException("Ontology " + ontologyId +
					"\nalready loaded but doesn't contain definition for\n" +
					elementId + ".");
		}

		this.ontoStore.addOntology(new OntologyShell(result));

		element = this.ontoStore.findDefinition(elementId, elementClass);
		if (element == null) {
			throw new RestorationException("Ontology " + ontologyId +
					"\nloaded but doesn't contain definition for\n" + 
					elementId + ".");
		}

		return element;
	}

	/**
	 * @see OntoStore.jumpToDefinition
	 * Extends OntoStore.jumpToDefinition by trying to load the necessary ontology
	 * if the definition is not found in the ontoStore.
	 * @return false if the defining ontology was neither found nor loaded.
s	 */
	public OntologyElement jumpToDefinition(OntologyElement element)
	{
		if (element == null) {
			throw new IllegalArgumentException("Trying to jump to a null element.");
		}

		// First, search for the definition in the current ontology store.
		OntologyElement definition = this.ontoStore.jumpToDefinition(element); 
		if (definition != null) {
			return definition;
		}

		// The element is not defined in any ontology from the ontology store.
		// An external file/url must loaded. Start the saerch from the ontologies
		// which are imported by the ontologies in the store.
		List<Ontology> importedOntos = new ArrayList<Ontology>();
		for (OntologyShell shell: this.ontoStore.listOntologies()) {
			importedOntos.addAll(Util.listImportedOntos(shell.getOntology()));			
		}

		try {
			element = findDefinition(element.getIdentifier(), element.getClass(), importedOntos);
			return this.ontoStore.jumpToDefinition(element);
		}
		catch (RestorationException ex) {
			Messages.showWarning(ex.getMessage());
			return null;
		}
	}

}
