package org.infrawebs.ontology.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.views.properties.IPropertySource;
import org.infrawebs.ontology.OntoFilter;
import org.infrawebs.ontology.OntoStore;
import org.infrawebs.ontology.shells.OntologyShell;
import org.omwg.ontology.OntologyElement;

public class OntoPanes extends Composite
{
	private TabFolder tabFolder = null;
	private OntoStore ontoStore = null;
	private OntoFilter ontoFilter = null;

	public OntoPanes(Composite parent) {
		super(parent, SWT.NONE);
		this.setLayout(new FillLayout());
		this.tabFolder = new TabFolder(this, SWT.TOP);
		Font tabFont = new Font(null, "Arial", 9, SWT.BOLD);
		this.tabFolder.setFont(tabFont);
	}

	// Displays all ontologies found in the ontoloty store in separate panes.
	private void createPanes() {
		closePanes();
		OntologyShell[] ontologies = this.ontoStore.listOntologies();
		for (int i = 0; i < ontologies.length; i++) {
			createPane(ontologies[i]);
		}
	}

	// Closes all ontology panes.
	private void closePanes() {
		TabItem[] items = this.tabFolder.getItems();
		for (int i = 0; i < items.length; i++) {
			items[i].dispose();
		}
	}

	// Appends a new tab for the specified ontology
	// if the pane is not empty after the ontoFilter is applied.
	public OntoPane createPane(OntologyShell ontology) {
		OntoPane ontoPane = new OntoPane(this.tabFolder);
		ontoPane.setOntology(ontology, this.ontoFilter);
		// After applying the filter, the onto pane may be empty.
		if (ontoPane.isEmpty()) {
			ontoPane.dispose();
			return null;
		}
		// Create a new non-empty tab and select it.
		TabItem ontoTab = new TabItem(this.tabFolder, SWT.NONE);
		ontoTab.setText(ontology.getPrefix());
		ontoTab.setControl(ontoPane);
		this.tabFolder.setSelection(this.tabFolder.getItemCount() - 1);
		return ontoPane;
	}
	
	// Closes the selected pane only. TODO: Remove ontology from store.
	/*private void closeSelectedPane() {
		TabItem[] selection = tabFolder.getSelection();
		if (selection != null) {
			for (int i = 0; i < selection.length; i++) {
				selection[i].dispose();
			}
		}
	}*/
	
	public Object getSelection() {
		int index = this.tabFolder.getSelectionIndex();
		if (index >= 0) {
			TabItem item = this.tabFolder.getItems()[index];
			OntoPane ontoPane = (OntoPane)item.getControl(); 
			return ontoPane.getSelection();
		}
		return null;
	}

    public IPropertySource getSelectionPropertiesSource() {
        int index = this.tabFolder.getSelectionIndex();
        if (index >= 0) {
            TabItem item = this.tabFolder.getItems()[index];
            OntoPane ontoPane = (OntoPane)item.getControl(); 
            return ontoPane.getOntology().getPropertySource();
        }
        return null;
    }

	// Selects the tab from the tab folder and the first item
	// from its tree which contains this particular ontology element.
	public void jumpToElement(OntologyElement element) {
		OntologyShell ao = this.ontoStore.getOntologyContaining(element);
		if (ao == null) {
			return;
		}

		TabItem[] items = this.tabFolder.getItems();
		for (int i = 0; i < items.length; i++) {
			OntoPane ontoPane = (OntoPane)items[i].getControl();
			if (ao == ontoPane.getOntology()) {
				this.tabFolder.setSelection(i);
				ontoPane.jumpToElement(element);
				break;
			}
		}
	}

	// Updates the attributes of all concepts. This is done to add or remove attibutes
	// from newly opened/closed ontologies in the Store.
	public void updateAllConcepts() {

		TabItem[] items = this.tabFolder.getItems();
		for (int i = 0; i < items.length; i++) {
			OntoPane ontoPane = (OntoPane)items[i].getControl();
			ontoPane.updateAllConcepts();
		}
	}
	
	public boolean isEmpty() {
		return (this.tabFolder.getTabList().length == 0);
	}

	public OntoFilter getOntoFilter() {
		return this.ontoFilter;
	}

	public void setOntoFilter(OntoFilter ontoFilter) {
		this.ontoFilter = ontoFilter;
	}

	public OntoStore getOntoStore() {
		return this.ontoStore;
	}

	public void setOntoStore(OntoStore ontoStore) {
		this.ontoStore = ontoStore;
		createPanes();
	}

}
